Package: wordvector
Type: Package
Title: Word and Document Vector Models
Version: 0.6.2
Authors@R: c(
    person('Kohei', 'Watanabe', role = c('aut', 'cre', 'cph'), email = 'watanabe.kohei@gmail.com', comment = c(ORCID = '0000-0001-6519-5265')), 
    person('Jan', 'Wijffels', role = 'aut', email = 'jwijffels@bnosac.be', comment = "Original R code"), 
    person('BNOSAC', role = 'cph', comment = "Original R code"),
    person('Max', 'Fomichev', role = c('ctb', 'cph'), comment = "Original C++ code"))
Maintainer: Kohei Watanabe <watanabe.kohei@gmail.com>
Description: Create dense vector representation of words and documents using 'quanteda'. Implements Word2vec (Mikolov et al., 2013) <doi:10.48550/arXiv.1310.4546>, Doc2vec (Le & Mikolov, 2014) <doi:10.48550/arXiv.1405.4053> and Latent Semantic Analysis (Deerwester et al., 1990) <doi:10.1002/(SICI)1097-4571(199009)41:6%3C391::AID-ASI1%3E3.0.CO;2-9>.
URL: https://github.com/koheiw/wordvector
License: Apache License (>= 2.0)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 3.5.0)
Imports: quanteda (>= 4.1.0), methods, stringi, Matrix, proxyC,
        RSpectra, irlba, rsvd
Suggests: testthat, word2vec, spelling
LinkingTo: Rcpp, quanteda
Language: en-US
LazyData: true
NeedsCompilation: yes
Packaged: 2026-04-06 06:08:57 UTC; watan
Author: Kohei Watanabe [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-6519-5265>),
  Jan Wijffels [aut] (Original R code),
  BNOSAC [cph] (Original R code),
  Max Fomichev [ctb, cph] (Original C++ code)
Repository: CRAN
Date/Publication: 2026-04-06 22:20:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-08 22:32:31 UTC; unix
Archs: wordvector.so.dSYM

Package: wordvector
Type: Package
Title: Word and Document Vector Models
Version: 0.6.3
Authors@R: c(
    person('Kohei', 'Watanabe', role = c('aut', 'cre', 'cph'), email = 'watanabe.kohei@gmail.com', comment = c(ORCID = '0000-0001-6519-5265')), 
    person('Jan', 'Wijffels', role = 'aut', email = 'jwijffels@bnosac.be', comment = "Original R code"), 
    person('BNOSAC', role = 'cph', comment = "Original R code"),
    person('Max', 'Fomichev', role = c('ctb', 'cph'), comment = "Original C++ code"))
Maintainer: Kohei Watanabe <watanabe.kohei@gmail.com>
Description: Create dense vector representation of words and documents using 'quanteda'. Implements Word2vec (Mikolov et al., 2013) <doi:10.48550/arXiv.1310.4546>, Doc2vec (Le & Mikolov, 2014) <doi:10.48550/arXiv.1405.4053> and Latent Semantic Analysis (Deerwester et al., 1990) <doi:10.1002/(SICI)1097-4571(199009)41:6%3C391::AID-ASI1%3E3.0.CO;2-9>.
URL: https://github.com/koheiw/wordvector,
        https://koheiw.github.io/wordvector/
License: Apache License (>= 2.0)
Encoding: UTF-8
Depends: R (>= 3.5.0)
Imports: quanteda (>= 4.1.0), methods, stringi, Matrix, proxyC,
        RSpectra, irlba, rsvd
Suggests: testthat, word2vec, spelling, rmarkdown, knitr
LinkingTo: Rcpp, quanteda
Language: en-US
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-28 06:48:43 UTC; watan
Author: Kohei Watanabe [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-6519-5265>),
  Jan Wijffels [aut] (Original R code),
  BNOSAC [cph] (Original R code),
  Max Fomichev [ctb, cph] (Original C++ code)
Repository: CRAN
Date/Publication: 2026-07-28 10:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.matrix,textmodel_doc2vec)
S3method(as.matrix,textmodel_docvector)
S3method(as.matrix,textmodel_word2vec)
S3method(as.matrix,textmodel_wordvector)
S3method(as.textmodel_doc2vec,dfm)
S3method(print,textmodel_doc2vec)
S3method(print,textmodel_docvector)
S3method(print,textmodel_word2vec)
S3method(print,textmodel_wordvector)
S3method(textmodel_doc2vec,tokens)
S3method(textmodel_lsa,dfm)
S3method(textmodel_lsa,tokens)
S3method(textmodel_word2vec,tokens)
export(analogy)
export(as.textmodel_doc2vec)
export(perplexity)
export(probability)
export(similarity)
export(textmodel_doc2vec)
export(textmodel_lsa)
export(textmodel_word2vec)
import(quanteda)
importFrom(methods,as)
importFrom(quanteda,dfm_weight)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(wordvector)

library(quanteda)
library(stringi)

dat <- readRDS('~/yahoo-news.RDS')
dat <- dat[order(dat$time),]
dat$head <- stri_replace_all_regex(dat$head, "\\s+", " ") %>% 
    stri_trans_nfkc() %>% 
    stri_trim()
dat$body <- stri_replace_all_regex(dat$body, "\\s+", " ") %>% 
    stri_trans_nfkc() %>% 
    stri_trim()
dat$time <- NULL

# # both head and body
dat$text <- paste0(dat$head, ". ", dat$body)
dat$head <- NULL
dat$body <- NULL
dat <- subset(dat, !duplicated(text))
corp <- corpus(dat, text_field = "text", docid_field = "tid")

# # only body
# dat <- subset(dat, !duplicated(body))
# corp <- corpus(dat, text_field = "body", docid_field = "tid")

# set.seed(1234)
# data_corpus_news <- corpus_sample(corp, 20000)
# usethis::use_data(data_corpus_news)
# 
# set.seed(1234)
# data_corpus_news2015 <- corpus_subset(corp, lubridate::year(date) == 2015) %>%
#     corpus_sample(20000)
# usethis::use_data(data_corpus_news2015)

set.seed(1234)
data_corpus_news2014 <- corpus_subset(corp, lubridate::year(date) == 2014) %>%
    corpus_sample(20000)
usethis::use_data(data_corpus_news2014)

library(quanteda)
library(wordvector)
options(wordvector_threads = 8)

seed <- LSX::seedwords("sentiment")

corp <- data_corpus_news2014
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>%
    tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>%
    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                  padding = TRUE) %>%
    tokens_tolower()

wov <- textmodel_word2vec(toks, dim = 100, type = "sg", min_count = 5, verbose = TRUE, iter = 10)
dov <- textmodel_doc2vec(toks, dim = 100, type = "dbow", min_count = 5, verbose = TRUE, iter = 10)

doc <- probability(dov, seed, layer = "documents")[,1]
print(toks[head(doc)])
print(toks[tail(doc)])

cor(probability(wov, seed, layer = "words", mode = "numeric")[,1],
    probability(dov, seed, layer = "words", mode = "numeric")[,1])

lss0 <- LSX::as.textmodel_lss(wov, seed, spatial = FALSE)
lss <- LSX:::as.textmodel_lss(dov, seed, spatial = FALSE)
pred0 <- predict(lss0, newdata = dfm(toks, remove_padding = TRUE), rescale = FALSE,
                 min_n = 40)
pred <- predict(lss, newdata = dfm(toks, remove_padding = TRUE), rescale = FALSE,
                 min_n = 40)
pred2 <- probability(dov, seed, layer = "documents", mode = "numeric")[,1] / length(seed)

dat <- data.frame(pred0, pred, pred2, pred3 = pred + pred2, text = corp[])

plot(dat[,1:4])
cor(dat[,1:4])
View(dat)
head(dat)

library(udpipe)
library(word2vec)

data(brussels_reviews, package = "udpipe")
x <- subset(brussels_reviews, language == "nl")
x <- tolower(x$feedback)

model <- word2vec::word2vec(x = x, dim = 15, iter = 20)
emb <- as.matrix(model)

pred <- predict(model, emb["bus",], type = "nearest", top_n = 10)
pred

# similarity in the library
cross <- rowSums(sqrt(crossprod(t(emb), emb["bus",]) / ncol(emb)))
head(sort(cross, decreasing = TRUE))

# cosine similarity 
cosine <- Matrix::rowSums(proxyC::simil(emb, emb["bus",,drop = FALSE]))
head(sort(cosine, decreasing = TRUE))

# they are very similar but different
cor(cross, cosine, use = "pair")
cor(cross, cosine, use = "pair", method = "spearman")    

# Test script for memory errors on linux
# 
# sudo apt-get install valgrind
# 
# R -d "valgrind --tool=memcheck --leak-check=full" --vanilla < tests/misc/valgrind.R
# 
# See https://cran.r-project.org/doc/manuals/r-devel/R-exts.html#Using-Valgrind
#--------------------------------------------------------------------------------

library(quanteda)
library(wordvector)

toks <- tokens(data_corpus_news2014, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords(), padding = TRUE) %>% 
    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                  padding = TRUE) %>% 
    tokens_tolower()

cb1 <- word2vec(toks, dim = 10, type = "cbow", use_ns = TRUE, iter = 5, verbose = TRUE, ns_size = 20)

cb2 <- word2vec(toks, dim = 10, type = "cbow", use_ns = FALSE, iter = 5, verbose = TRUE)

sg1 <- word2vec(toks, dim = 10, type = "skip-gram", use_ns = TRUE, iter = 5, verbose = TRUE)

sg2 <- word2vec(toks, dim = 10, type = "skip-gram", use_ns = FALSE, iter = 5, verbose = TRUE)

library(quanteda)
library(Matrix)
library(doc2vec)
library(wordvector)

dat <- data.frame(doc_id = docnames(data_corpus_news2014),
                  text = as.character(data_corpus_news2014))
dat2 <- head(dat, 100)
dat2$doc_id <- paste0(dat2$doc_id, "_copy")
dat3 <- rbind(dat, dat2)
rownames(dat3) <- dat3$doc_id

corp <- corpus(dat3)
toks <- tokens(corp)
dfmt <- dfm(toks, remove_padding = TRUE)

# doc2vec package -----------------------------

d2v <- paragraph2vec(dat3, dim = 50, threads = 8, type = "PV-DBOW", trace = FALSE, iter = 10)
mat_d2v <- as.matrix(d2v, which = "docs", normalize = FALSE)
hist(mat_d2v["4362315",, drop = TRUE])

sim_d2v <- proxyC::simil(
    mat_d2v,
    mat_d2v["4362315",, drop = FALSE]
)

sim_d2v <- proxyC::simil(
    mat_d2v,
    mat_d2v["4102068",, drop = FALSE]
)

hist(rowSums(sim_d2v))
tail(sort(s <- rowSums(sim_d2v)))
#print(tail(dat3[order(s),]))

sim_d2v_all <- proxyC::simil(
    mat_d2v,
    mat_d2v[tail(dat3$doc_id, 100),, drop = FALSE]
)
mean(diag(sim_d2v_all)) - mean(sim_d2v_all)

# wordvector package -------------------------

options(wordvector_threads = 8)
wdv <- textmodel_doc2vec(toks, dim = 50, type = "dbow", min_count = 5, verbose = FALSE, iter = 10,
                         tolower = FALSE, alpha = 0.05)
mat_wdv <- as.matrix(wdv, layer = "documents", normalize = FALSE)
hist(mat_wdv["4362315",, drop = TRUE])

sim_wdv <- proxyC::simil(
    mat_wdv,
    mat_wdv["4362315",, drop = FALSE]
)

sim_wdv <- proxyC::simil(
    mat_wdv,
    mat_wdv["4102068",, drop = FALSE]
)

hist(rowSums(sim_wdv))
tail(sort(s <- rowSums(sim_wdv)))
#tail(dat3[order(s),])

sim_wdv_all <- proxyC::simil(
    mat_wdv,
    mat_wdv[tail(dat3$doc_id, 100),, drop = FALSE]
)
mean(diag(sim_wdv_all)) - mean(sim_wdv_all)

# -------------------------

# a <- rownames(as.matrix(d2v, which = "words", normalize = FALSE))
# b <- rownames(as.matrix(wdv, layer = "words", normalize = FALSE))
# length(a)
# length(b)
# length(intersect(a, b)) / length(union(a, b))

plot(rowSums(sim_wdv), rowSums(sim_d2v))
cor(rowSums(sim_wdv), rowSums(sim_d2v))

hist(as.matrix(wdv, layer = "words", normalize = FALSE)[1,])
hist(wdv$values$word[,1])
hist(wdv$weights[,1])
       

#' Doc2vec model
#' 
#' Train a doc2vec model (Le & Mikolov, 2014) using a [quanteda::tokens] object.
#' @export
#' @param type the architecture of the model; either "dm" (distributed memory) or 
#'   "dbow" (distributed bag-of-words).
#' @param window the size of the window for context words. Ignored when `type = "dbow"` as
#'   its context window is the entire document (sentence or paragraph).
#' @inheritParams textmodel_word2vec
#' @return 
#' Returns a textmodel_doc2vec object with matrices for word and document vector 
#' values, [quanteda::docvars] and [quanteda::ntoken] of `x`. Other elements are 
#' the same as [wordvector::textmodel_word2vec]. 
#' @references 
#'   Le, Q. V., & Mikolov, T. (2014). Distributed Representations of Sentences and 
#'   Documents (No. arXiv:1405.4053). arXiv. https://doi.org/10.48550/arXiv.1405.4053
textmodel_doc2vec <- function(x, dim = 50, type = c("dm", "dbow"), 
                              min_count = 5, window = 5, 
                              iter = 10, alpha = 0.05, model = NULL, 
                              use_ns = TRUE, ns_size = 5, sample = 0.001, tolower = TRUE,
                              include_data = FALSE, verbose = FALSE, ...) {
    UseMethod("textmodel_doc2vec")
}

#' @export
#' @method textmodel_doc2vec tokens
textmodel_doc2vec.tokens <- function(x, dim = 50, type = c("dm", "dbow"), 
                                     min_count = 5, window = 5, 
                                     iter = 10, alpha = 0.05, model = NULL, 
                                     use_ns = TRUE, ns_size = 5, sample = 0.001, tolower = TRUE,
                                     include_data = FALSE, verbose = FALSE, ...) {
    
    type <- match.arg(type)
    wordvector(x, dim, type, TRUE, min_count, window, iter, alpha, model, 
               use_ns, ns_size, sample, tolower, include_data, verbose, ...)
    
}

#' @rdname as.matrix
#' @export
as.matrix.textmodel_doc2vec <- function(x, normalize = TRUE, 
                                        layer = c("documents", "words"), 
                                        group = FALSE, ...) {
    
    x <- upgrade_pre06(x)
    normalize <- check_logical(normalize)
    layer <- match.arg(layer)
    group <- check_logical(group)
    
    if (layer == "words") {
        result <- x$values$word
    } else {
        # TODO: add grouping by docid
        if (group) {
            lis <- split.data.frame(x$values$doc, x$docvars[["docid_"]])
            result <- t(sapply(lis, colMeans))
        } else {
            result <- x$values$doc 
        }
    }
    if (is.null(result))
        stop("x does not have the layer for ", layer)
    if (normalize) {
        v <- sqrt(rowSums(result ^ 2) / ncol(result))
        result <- result / v
    }
    return(result) 
}

require(quanteda)
require(wordvector)

corp <- data_corpus_news2014
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>%
   tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>%
   tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                 padding = TRUE) %>%
   tokens_tolower()

toks0 <- head(toks, 5000)
toks1 <- tail(toks, 5000)

test_that("textmodel_word2vec() works", {
    
    skip_on_cran()
    
    wov0 <- textmodel_word2vec(toks0, dim = 50, type = "cbow")
    wov1 <- textmodel_word2vec(toks1, dim = 50, type = "cbow")
    wov2 <- textmodel_word2vec(toks1, dim = 50, type = "cbow", model = wov0)
    
    expect_false(identical(rownames(wov1$values$word), 
                           rownames(wov0$values$word)))
    expect_false(identical(rownames(wov2$values$word), 
                           rownames(wov0$values$word)))
    expect_true(identical(rownames(wov1$values$word), 
                          rownames(wov2$values$word)))
    
    # without model
    f1 <- intersect(rownames(wov0$values$word), 
                    rownames(wov1$values$word))
    sim1 <- Matrix::diag(proxyC::simil(wov0$values$word[f1,], 
                                       wov1$values$word[f1,], diag = TRUE))
    expect_true(
        median(sim1) < 0.20
    )
    
    # with model
    f2 <- intersect(rownames(wov0$values$word), 
                    rownames(wov2$values$word))
    sim2 <- Matrix::diag(proxyC::simil(wov0$values$word[f2,], 
                                       wov2$values$word[f2,], diag = TRUE))
    expect_true(
        median(sim2) > 0.70
    )
    
    expect_error(
        textmodel_word2vec(toks1, dim = 50, type = "cbow", model = list()),
        "model must be a trained textmodel_word2vec"
    )
    
    expect_error(
        textmodel_word2vec(toks1, dim = 50, type = "dbow", model = wov0),
        "'arg' should be one of \"cbow\", \"sg\", \"dm\""
    )
    
    expect_warning(
        textmodel_word2vec(toks1, dim = 25, type = "cbow", model = wov0),
        "dim, type and use_na are overwritten by the pre-trained model"
    )
    
    expect_warning(
        textmodel_word2vec(toks1, dim = 25, type = "sg", model = wov0),
        "dim, type and use_na are overwritten by the pre-trained model"
    )
})


test_that("textmodel_doc2vec() works", {
    
    skip_on_cran()
    
    dov0 <- textmodel_doc2vec(toks0, dim = 50, type = "dm")
    dov1 <- textmodel_doc2vec(toks1, dim = 50, type = "dm")
    dov2 <- textmodel_doc2vec(toks1, dim = 50, type = "dm", model = dov0)
    
    # word layer
    expect_false(identical(rownames(dov1$values$word), 
                           rownames(dov0$values$word)))
    expect_false(identical(rownames(dov2$values$word), 
                           rownames(dov0$values$word)))
    expect_true(identical(rownames(dov1$values$word), 
                          rownames(dov2$values$word)))
    
    # document layer
    expect_false(identical(rownames(dov1$values$doc), 
                           rownames(dov0$values$doc)))
    expect_false(identical(rownames(dov2$values$doc), 
                           rownames(dov0$values$doc)))
    expect_true(identical(rownames(dov1$values$doc), 
                          rownames(dov2$values$doc)))
    
    # without model
    f1 <- intersect(rownames(dov0$values$word), 
                    rownames(dov1$values$word))
    sim1 <- Matrix::diag(proxyC::simil(dov0$values$word[f1,], 
                                       dov1$values$word[f1,], diag = TRUE))
    expect_true(
        median(sim1) < 0.20
    )
    
    # with model
    f2 <- intersect(rownames(dov0$values$word), 
                    rownames(dov2$values$word))
    sim2 <- Matrix::diag(proxyC::simil(dov0$values$word[f2,], 
                                       dov2$values$word[f2,], diag = TRUE))
    expect_true(
        median(sim2) > 0.70
    )
    
    expect_error(
        textmodel_doc2vec(toks1, dim = 50, type = "dm", model = list()),
        "model must be a trained textmodel_word2vec or textmodel_doc2vec"
    )
    
    expect_warning(
        textmodel_doc2vec(toks1, dim = 10, type = "dm", model = dov0),
        "dim, type and use_na are overwritten by the pre-trained model"
    )
    
    expect_warning(
        textmodel_doc2vec(toks1, dim = 50, type = "dbow", model = dov0),
        "dim, type and use_na are overwritten by the pre-trained model"
    )
})

#' Convert formula to named character vector
#' 
#' Convert a formula to a named character vector in analogy tasks.
#' @param formula a [formula] object that defines the relationship between words 
#'   using `+` or `-` operators.
#' @export
#' @seealso [similarity()]
#' @importFrom utils head tail
#' @return a named character vector to be passed to [similarity()].
#' @examples
#' analogy(~ berlin - germany + france)
#' analogy(~ quick - quickly + slowly)
analogy <- function(formula) {
    
    if (!identical(class(formula), "formula"))
        stop("formula must be a formula object")
    
    f <- tail(as.character(formula), 1)
    match <- stringi::stri_match_all_regex(f, "([+-])?\\s*(\\w+)")[[1]]
    match[,2] <- stringi::stri_trim(match[,2])
    match[,2][is.na(match[,2])] <- "+"
    res <- numeric()
    for (i in seq_len(nrow(match))) {
        m <- match[i,]
        if (m[2] == "-") {
            res <- c(res, structure(-1.0, names = m[3]))
        } else if (m[2] == "+") {
            res <- c(res, structure(1.0, names = m[3]))
        }
    }
    return(res)
}

#' Compute similarity between word or document vectors
#' 
#' Compute the cosine similarity between word vectors for selected words.
#' @param x a `textmodel_wordvector` object.
#' @param targets words or documents for which similarity is computed.
#' @param layer the layer based on which similarity is computed. This must be "documents" 
#'   when `targets` are document names.
#' @param mode specify the type of resulting object.
#' @return a `matrix` of cosine similarity scores when `mode = "numeric"` or of 
#'   words sorted in descending order by the similarity scores when `mode = "character"`.
#'   When `targets` is a named numeric vector, word (or document) vectors are weighted and summed 
#'   before computing similarity scores.
#' @export
#' @seealso [probability()]
similarity <- function(x, targets, layer = c("words", "documents"),
                       mode = c("character", "numeric")) {
    
    layer <- match.arg(layer)
    mode <- ifelse(mode == "words", "character", mode) # for < v0.6.0
    mode <- ifelse(mode == "values", "numeric", mode) # for < v0.6.0
    mode <- match.arg(mode)
    emb1 <- as.matrix(x, layer = layer, normalize = TRUE)
    
    if (!"textmodel_wordvector" %in% class(x))
        stop("x must be a textmodel_wordvector object")
    
    if (is.character(targets)) {
        targets <- structure(rep(1.0, length(targets)), names = targets)
        weighted <- FALSE
    } else if (is.numeric(targets)) {
        if (is.null(names(targets)))
            stop("targets must be named")
        weighted <- TRUE 
    } else {
        stop("targets must be a character vector or a named numeric vector")
    }
    b <- names(targets) %in% rownames(emb1)
    if (sum(!b) == 1) {
        warning(paste0('"', names(targets[!b]), '"',  collapse = ", "),  ' is not found')
    } else if (sum(!b) > 1) {
        warning(paste0('"', names(targets[!b]), '"',  collapse = ", "),  ' are not found')
    }
    targets <- targets[b]
    if (weighted) {
        emb2 <- rbind(colSums(emb1[names(targets),, drop = FALSE] * targets))
    } else {
        emb2 <- emb1[names(targets),, drop = FALSE]
    }
    res <- as.matrix(proxyC::simil(emb1, emb2, use_nan = TRUE))
    if (ncol(res) == 0) {
        res <- matrix(nrow = 0, ncol = 0)
    } else {
        if (mode == "character") {
            res <- apply(res, 2, function(v) {
                names(sort(v, decreasing = TRUE))
            })
        }
    }
    return(res)
}

#' Compute probability of words
#'
#' Compute the probability of words given other words.
#' @param x a trained `textmodel_wordvector` object.
#' @param targets words for which probabilities are computed.
#' @param layer the layer based on which probabilities are computed.
#' @param mode specify the type of resulting object.
#' @param ... passed to `as.matrix()`.
#' @return a matrix of words or documents sorted in descending order by the probability 
#'   scores when `mode = "character"`; a matrix of the probability scores when `mode = "numeric"`.
#'   When `targets` is a named numeric vector, probability scores are weighted by
#'   the values.
#' @export
#' @seealso [similarity()]
probability <- function(x, targets, layer = c("words", "documents"),
                        mode = c("character", "numeric"), ...) {
    
    layer <- match.arg(layer)
    mode <- ifelse(mode == "words", "character", mode) # for < v0.6.0
    mode <- ifelse(mode == "values", "numeric", mode) # for < v0.6.0
    mode <- match.arg(mode)
    
    if ("textmodel_word2vec" %in% class(x) && layer == "documents")
        stop("textmodel_word2vec does not have the layer for documents")
    
    if (!"textmodel_wordvector" %in% class(x))
        stop("x must be a textmodel_wordvector object")
    
    if (is.null(x$weights))
        stop("x must be a trained textmodel_wordvector object")
    
    if (x$normalize)
        stop("x must be trained with normalize = FALSE")
    
    if (is.character(targets)) {
        targets <- structure(rep(1.0, length(targets)), names = targets)
        weighted <- FALSE
    } else if (is.numeric(targets)) {
        if (is.null(names(targets)))
            stop("targets must be named")
        weighted <- TRUE 
    } else {
        stop("targets must be a character vector or a named numeric vector")
    }

    b <- names(targets) %in% rownames(x$weights)
    if (sum(!b) == 1) {
        warning(paste0('"', names(targets[!b]), '"',  collapse = ", "),  ' is not found')
    } else if (sum(!b) > 1) {
        warning(paste0('"', names(targets[!b]), '"',  collapse = ", "),  ' are not found')
    }
    targets <- targets[b]
    
    values <- as.matrix(x, layer = layer, normalize = FALSE, ...)
    e <- exp(tcrossprod(values, x$weights[names(targets),, drop = FALSE]))
    prob <- e / (e + 1) # sigmoid function
    
    if (weighted) {
        res <- prob %*% diag(targets)
        colnames(res) <- names(targets)
        res <- cbind(rowSums(res))
    } else {
        res <- prob
    }
    
    if (ncol(res) == 0) {
        res <- matrix(nrow = 0, ncol = 0)
    } else {
        if (mode == "character") {
            res <- apply(res, 2, function(v) {
                names(sort(v, decreasing = TRUE))
            })
        }
    }
    return(res)
}

#' \[experimental\] Compute perplexity of a model
#'
#' Compute the perplexity of a trained word2vec model with data.
#' @param x a trained `textmodel_wordvector` object.
#' @param data a [quanteda::tokens] or [quanteda::dfm]; the probabilities of words are 
#'    tested against occurrences of words in it.
#' @inheritParams probability
#' @export
#' @keywords internal
perplexity <- function(x, targets, data, layer = c("words", "documents")) {
    
    x <- upgrade_pre06(x)
    layer <- match.arg(layer)
    
    if (!is.character(targets))
        stop("targets must be a character vector")
    
    if (!is.tokens(data) && !is.dfm(data))
        stop("data must be a tokens or dfm")
    data <- dfm(data, remove_padding = TRUE, tolower = x$tolower)

    p <- probability(x, targets, mode = "numeric", layer = layer)
    if (layer == "words") {
        pred <- dfm_match(data, rownames(p)) %*% p
    } else {
        if (!all(docnames(data) %in% rownames(p)))
            stop("x must be trained on the documents in data")
        pred <- p[docnames(data),,drop = FALSE]
    }
    pred <- pred / rowSums(pred)
        
    data <- dfm_match(data, colnames(pred))
    data <- Matrix::mat2triplet(data)
    exp(-sum(data$x * log(pred[cbind(data$i, data$j)])) / sum(data$x))
}

get_threads <- function() {
    
    # respect other settings
    default <- c("tbb" = as.integer(Sys.getenv("RCPP_PARALLEL_NUM_THREADS")),
                 "omp" = as.integer(Sys.getenv("OMP_THREAD_LIMIT")),
                 "max" = cpp_get_max_thread())
    default <- unname(min(default, na.rm = TRUE))
    suppressWarnings({
        value <- as.integer(getOption("wordvector_threads", default))
    })
    if (length(value) != 1 || is.na(value)) {
        stop("wordvector_threads must be an integer")
    }
    return(value)
}

upgrade_pre06 <- function(x) {
    
    if (is.null(x$tolower)) {
        x$tolower <- TRUE
    }
    if (is.numeric(x$type)) {
        x$type <- c("cbow", "sg")[x$type]
    }
    if (is.list(x$values))
        return(x)
    if (identical(class(x), "textmodel_wordvector")) {
        x$values <- list(word = x$values)
        class(x) <- c("textmodel_word2vec", "textmodel_wordvector")
    } else if (identical(class(x), "textmodel_docvector")) {
        x$values  <- list(doc = x$values)
        class(x) <- c("textmodel_doc2vec", "textmodel_wordvector")
    }
    return(x)
}

is_word2vec <- function(x) {
    identical(class(x), c("textmodel_word2vec", "textmodel_wordvector"))
}

is_doc2vec <- function(x) {
    identical(class(x), c("textmodel_doc2vec", "textmodel_wordvector"))
}

check_word2vec <- function(x) {
    if (is_word2vec(x)) {
        return(x)
    } else {
        stop("model must be a trained textmodel_word2vec")
    }
}

check_doc2vec <- function(x) {
    if (is_doc2vec(x)) {
        return(x)
    } else {
        stop("model must be a trained textmodel_doc2vec")
    }
}

check_model <- function(x, allow = c("word2vec", "doc2vec", "lsa")) {
    allow <- match.arg(allow, several.ok = TRUE)
    m <- paste0("textmodel_", allow)
    if (any(class(x)[1] == m & class(x)[2] == "textmodel_wordvector")) {
        return(x)
    } else {
        stop("model must be a trained ", paste(m, collapse = " or "))
    }
}

library(quanteda)
library(wordvector)
options(wordvector_threads = 2)

corp <- head(data_corpus_inaugural, 59) %>% 
    corpus_reshape()

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE,
               concatenator = " ") %>% 
    tokens_remove(stopwords(), padding = TRUE) %>% 
    tokens_compound(data_dictionary_LSD2015, keep_unigrams = TRUE)

test_that("textmodel_doc2vec works", {
    
    # DM
    expect_output(
        dov1 <- textmodel_doc2vec(toks, dim = 50, iter = 5, min_count = 2, verbose = TRUE),
        "Training distributed memory model with 50 dimensions"
    )
    expect_equal(dov1$type, "dm")
    expect_false(dov1$normalize)
    expect_equal(
        names(dov1),
        c("values", "weights", "type", "dim", "frequency", "window",  "iter", "alpha", 
          "use_ns", "ns_size", "sample", "normalize",  "min_count", "tolower",
          "concatenator", "docvars", "ntoken", "call", "version")
    )
    expect_equal(
        dim(dov1$values$word), c(5363L, 50L)
    )
    expect_equal(
        dim(dov1$values$doc), c(5296L, 50L)
    )
    expect_output(
        print(dov1),
        paste(
            "",
            "Call:",
            "textmodel_doc2vec(x = toks, dim = 50, min_count = 2, iter = 5, ",
            "    verbose = TRUE)",
            "",
            "50 dimensions; 5,296 documents.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(dov1))), 
        class(dov1)
    )
    
    expect_equal(
        rownames(probability(dov1, c("good", "bad"), layer = "words", mode = "numeric")),
        rownames(dov1$values$word)
    )
    
    expect_equal(
        rownames(probability(dov1, c("good", "bad"), layer = "documents", mode = "numeric")),
        rownames(dov1$values$doc)
    )
    
    expect_equal(
        rownames(probability(dov1, c("good", "bad"), layer = "documents", mode = "numeric",
                             group = TRUE)),
        levels(dov1$docvars$docid_)
    )
    
    expect_identical(
        dov1$ntoken,
        ntoken(tokens_trim(tokens_tolower(toks), min_termfreq = 2), remove_padding = TRUE)
    )
    
    expect_true(
        is.numeric(perplexity(dov1, c("good", "bad"), toks, layer = "documents"))
    )
    
    expect_true(
        is.numeric(perplexity(dov1, c("good", "bad"), toks, layer = "words"))
    )
    
    expect_error(
        is.numeric(perplexity(dov1, c("good", "bad"), tokens("a b c"), layer = "documents")),
        "x must be trained on the documents in data"
    )
    
    # DBOW
    expect_output(
        dov2 <- textmodel_doc2vec(toks, dim = 50, type = "dbow", iter = 5, min_count = 2, verbose = TRUE),
        "Training distributed BOW model with 50 dimensions"
    )
    expect_equal(dov2$type, "dbow")
    expect_false(dov2$normalize)
    expect_equal(
        names(dov2),
        c("values", "weights", "type", "dim", "frequency", "window",  "iter", "alpha", 
          "use_ns", "ns_size", "sample", "normalize",  "min_count", "tolower",
          "concatenator", "docvars", "ntoken", "call", "version")
    )
    expect_null(
        dov2$values$word
    )
    expect_equal(
        dim(dov2$values$doc), c(5296L, 50L)
    )
    expect_output(
        print(dov2),
        paste(
            "",
            "Call:",
            "textmodel_doc2vec(x = toks, dim = 50, type = \"dbow\", min_count = 2, ", 
            "    iter = 5, verbose = TRUE)",
            "",
            "50 dimensions; 5,296 documents.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(dov2))), 
        class(dov2)
    )
    
    expect_error(
        probability(dov2, c("good", "bad"), layer = "words", mode = "numeric"),
        "x does not have the layer for words"
    )
    
    expect_equal(
        rownames(probability(dov2, c("good", "bad"), layer = "documents", mode = "numeric")),
        rownames(dov2$values$doc)
    )
    
    expect_equal(
        rownames(probability(dov2, c("good", "bad"), layer = "documents", mode = "numeric",
                             group = TRUE)),
        levels(dov2$docvars$docid_)
    )
    
    expect_identical(
        dov2$ntoken,
        ntoken(tokens_trim(tokens_tolower(toks), min_termfreq = 2), remove_padding = TRUE)
    )
    
    expect_true(
        is.numeric(perplexity(dov2, c("good", "bad"), toks, layer = "documents"))
    )
    
    expect_error(
        is.numeric(perplexity(dov2, c("good", "bad"), toks, layer = "words")),
        "x does not have the layer for words"
    )

    expect_error(
        is.numeric(perplexity(dov2, c("good", "bad"), tokens("a b c"), layer = "documents")),
        "x must be trained on the documents in data"
    )
})

test_that("textmodel_doc2vec works hierachical softmax", {
    
    skip_on_cran()
    
    # DM
    dov1 <- textmodel_doc2vec(head(toks, 1000), dim = 10, type = "dm", use_ns = FALSE)
    expect_equal(
        class(dov1), 
        c("textmodel_doc2vec", "textmodel_wordvector")
    )
    expect_false(
        dov1$use_ns
    )
    expect_equal(
        dov1$type, 
        "dm"
    )
    
    # DBOW
    dov2 <- textmodel_doc2vec(head(toks, 1000), dim = 10, type = "dbow", use_ns = FALSE)
    expect_equal(
        class(dov2), 
        c("textmodel_doc2vec", "textmodel_wordvector")
    )
    expect_false(
        dov2$use_ns
    )
    expect_equal(
        dov2$type, 
        "dbow"
    )
})

test_that("textmodel_doc2vec works with pre-trained models", {
    
    skip_on_cran()
    
    # Zero learning
    dov0_pre <- textmodel_doc2vec(toks, type = "dm")
    dov0 <- textmodel_doc2vec(toks, type = "dm", iter = 1, model = dov0_pre,
                              alpha = 0)
    
    expect_identical(as.matrix(dov0, layer = "words"), 
                     as.matrix(dov0_pre, layer = "words"))
    
    # DM
    dov1_pre <- textmodel_doc2vec(toks, type = "dm")
    dov1 <- textmodel_doc2vec(toks, type = "dm", iter = 5, model = dov1_pre)
    
    r <- cor(t(as.matrix(dov1_pre, layer = "words"))[,1:1000], 
             t(as.matrix(dov1, layer = "words"))[,1:1000])
    expect_gt(median(diag(r)), 0.9)
    
    r <- cor(t(as.matrix(dov1_pre, layer = "documents")),
             t(as.matrix(dov1, layer = "documents")))
    expect_gt(median(diag(r)), 0.8)
    
    # DBOW
    dov2_pre <- textmodel_doc2vec(toks, type = "dbow")
    dov2 <- textmodel_doc2vec(toks, type = "dbow", iter = 5, model = dov2_pre)
    
    r <- cor(t(as.matrix(dov2_pre, layer = "documents")), 
             t(as.matrix(dov2, layer = "documents")))
    expect_gt(median(diag(r)), 0.8)
    
    # errors
    expect_error(
        textmodel_doc2vec(toks, type = "dbow", iter = 1, model = list()),
        "model must be a trained textmodel_word2vec or textmodel_doc2vec", fixed = TRUE
    )
    expect_warning(
        textmodel_doc2vec(toks, type = "dbow", iter = 1, model = dov1_pre),
        "dim, type and use_na are overwritten by the pre-trained model", fixed = TRUE
    )
})

library(quanteda)
library(wordvector)

corp <- head(data_corpus_inaugural, 59) %>% 
    corpus_reshape()

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords(), padding = FALSE) %>% 
    tokens_tolower()

dfmt <- dfm(toks, remove_padding = TRUE)

set.seed(1234)
wov <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                          normalize = FALSE)
dov <- as.textmodel_doc2vec(dfmt, model = wov)

test_that("as.matrix works", {
    
    # word2vec
    expect_setequal(rownames(as.matrix(wov)), 
                    types(tokens_trim(tokens_tolower(toks), min_termfreq = 2)))
    expect_error(
        as.matrix(wov, layer = "documents"),
        "'arg' should be \"words\""
    )
    
    expect_false(
        identical(as.matrix(wov, normalize = TRUE), wov$values$word)
    )
    expect_true(
        identical(as.matrix(wov, normalize = FALSE), wov$values$word)
    )
    
    expect_identical(
        dim(as.matrix(wov, padding = TRUE)), c(5361L, 50L)
    )
    expect_identical(
        dim(as.matrix(wov, padding = FALSE)), c(5360L, 50L)
    )
    
    # doc2vec
    expect_setequal(rownames(as.matrix(dov)), 
                    docnames(dfmt))
    expect_setequal(rownames(as.matrix(dov, layer = "words")), 
                    featnames(dfm_trim(dfmt, min_termfreq = 2)))
    
    expect_false(
        identical(as.matrix(dov, normalize = TRUE), dov$values$doc)
    )
    expect_true(
        identical(as.matrix(dov, normalize = FALSE), dov$values$doc)
    )
    
    # doc2vec with group
    expect_setequal(
        rownames(as.matrix(dov, layer = "document", group = TRUE)),
        docnames(dfm_group(dfmt))
    )
    
    expect_error(
        as.matrix(dov, layer = "document", group = c(TRUE, FALSE)),
        "The length of group must be 1"
    )
})

test_that("analogy works", {
    
    expect_equal(
        analogy(~ us),
        c("us" = 1)
    )
    
    expect_equal(
        analogy(~ us),
        c("us" = 1)
    )
    
    expect_equal(
        analogy(~ people - us),
        c("people" = 1, "us" = -1)
    )
    
    expect_error(
        analogy("people"),
        "formula must be a formula object"
    )
})

test_that("similarity works", {
    
    # word2vec
    sim1 <- similarity(wov, "us", mode = "numeric")
    expect_true(is.matrix(sim1))
    expect_identical(
        dimnames(sim1),
        list(names(wov$frequency), "us")
    )
    
    sim2 <- similarity(wov, c("us", "people"), mode = "numeric")
    expect_true(is.matrix(sim2))
    expect_identical(
        dimnames(sim2),
        list(names(wov$frequency), c("us", "people"))
    )
    
    sim3 <- similarity(wov, "us", mode = "character")
    expect_true(is.matrix(sim3))
    expect_identical(
        sim3[1,],
        c("us" = "us")
    )
    expect_identical(
        dim(sim3),
        c(length(wov$frequency), 1L)
    )
    
    sim4 <- similarity(wov, c("us", "people"), mode = "character")
    expect_true(is.matrix(sim4))
    expect_identical(
        sim4[1,],
        c("us" = "us", "people" = "people")
    )
    expect_identical(
        dim(sim4),
        c(length(wov$frequency), 2L)
    )
    expect_warning(
        similarity(wov, c("xx", "yyy", "us"), mode = "numeric"),
        '"xx", "yyy" are not found'
    )
    expect_true(
        suppressWarnings(
        is.matrix(similarity(wov, c("xx", "yyy"), mode = "numeric"))
        )
    )
    expect_warning(
        similarity(wov, "xx", mode = "character"),
        '"xx" is not found'
    )
    expect_warning(
        similarity(wov, c("xx", "yyy", "us"), mode = "character"),
        '"xx", "yyy" are not found'
    )
    expect_true(
        suppressWarnings(
            is.matrix(similarity(wov, c("xx", "yyy"), mode = "character"))
        )
    )
    
    sim5 <- similarity(wov, c("us" = 1, "people" = -1), mode = "numeric")
    expect_equal(ncol(sim5), 1)
    expect_true(is.matrix(sim5))
    expect_identical(
        dimnames(sim5),
        list(names(wov$frequency), NULL)
    )
    
    sim6 <- similarity(wov, c("us" = 1, "people" = -1), mode = "character")
    expect_equal(ncol(sim6), 1)
    expect_true(is.matrix(sim6))
    expect_identical(
        dimnames(sim6),
        NULL
    )
    expect_error(
        similarity(wov, c(TRUE, FALSE), mode = "character"),
        "targets must be a character vector or a named numeric vector"
    )
    expect_error(
        similarity(wov, c(1, -1), mode = "character"),
        "targets must be named"
    )
    
    # doc2vec
    sim10 <- similarity(dov, c("us" = 1, "people" = -1), mode = "numeric")
    expect_equal(ncol(sim10), 1)
    expect_true(is.matrix(sim10))
    expect_identical(
        dimnames(sim10),
        list(names(wov$frequency), NULL)
    )
    
    sim11 <- similarity(dov, c("us" = 1, "people" = -1), mode = "character")
    expect_equal(ncol(sim11), 1)
    expect_true(is.matrix(sim11))
    expect_identical(
        dimnames(sim11),
        NULL
    )
    
    sim12 <- similarity(dov, c("2009-Obama.1", "2017-Trump.1"), 
                        layer = "documents", mode = "numeric")
    expect_equal(ncol(sim12), 2)
    expect_true(is.matrix(sim12))
    expect_identical(
        dimnames(sim12),
        list(docnames(toks), c("2009-Obama.1", "2017-Trump.1"))
    )
    
    sim13 <- similarity(dov, c("2009-Obama.1", "2017-Trump.1"), 
                        layer = "documents", mode = "character")
    expect_equal(ncol(sim13), 2)
    expect_true(is.matrix(sim13))
    expect_identical(
        dimnames(sim13),
        list(NULL, c("2009-Obama.1", "2017-Trump.1"))
    )
    expect_error(
        similarity(dov, c(TRUE, FALSE), mode = "character"),
        "targets must be a character vector or a named numeric vector"
    )
    expect_error(
        similarity(list(), c("us" = 1, "people" = -1)),
        "x must be a textmodel_wordvector object"
    )
})

test_that("probability works", {
    
    skip_on_cran()
    skip_on_os("mac") # for github action

    # word2vec
    prob1 <- probability(wov, "us", mode = "numeric")
    expect_true(all(prob1 <= 1.0))
    expect_true(all(prob1 >= 0.0))
    expect_true(is.matrix(prob1))
    expect_identical(
        dimnames(prob1),
        list(names(wov$frequency), "us")
    )
    
    prob2 <- probability(wov, c("us", "people"), mode = "numeric")
    expect_true(all(prob2 <= 1.0))
    expect_true(all(prob2 >= 0.0))
    expect_true(is.matrix(prob2))
    expect_identical(
        dimnames(prob2),
        list(names(wov$frequency), c("us", "people"))
    )
    
    prob3 <- probability(wov, "us", mode = "character")
    expect_true(is.matrix(prob3))
    expect_identical(
        prob3[1,],
        c("us" = "let")
    )
    expect_identical(
        dim(prob3),
        c(length(wov$frequency), 1L)
    )
    
    prob4 <- probability(wov, c("us", "people"), mode = "character")
    expect_true(is.matrix(prob4))
    expect_identical(
        prob4[1,],
        c("us" = "let", "people" = "american")
    )
    expect_identical(
        dim(prob4),
        c(length(wov$frequency), 2L)
    )
    expect_warning(
        probability(wov, c("xx"), mode = "numeric"),
        '"xx" is not found'
    )
    expect_warning(
        probability(wov, c("xx", "yyy", "us"), mode = "numeric"),
        '"xx", "yyy" are not found'
    )
    expect_true(
        suppressWarnings(
            is.matrix(probability(wov, c("xx", "yyy"), mode = "numeric"))
        )
    )
    expect_warning(
        probability(wov, c("xx", "yyy", "us"), mode = "character"),
        '"xx", "yyy" are not found'
    )
    expect_true(
        suppressWarnings(
            is.matrix(probability(wov, c("xx", "yyy"), mode = "character"))
        )
    )
    
    prob5 <- probability(wov, c("us" = 1, "people" = -1), mode = "numeric")
    expect_equal(ncol(prob5), 1)
    expect_true(is.matrix(prob5))
    expect_identical(
        dimnames(prob5),
        list(names(wov$frequency), NULL)
    )
    
    prob6 <- probability(wov, c("us" = 1, "people" = -1), mode = "character")
    expect_equal(ncol(prob6), 1)
    expect_true(is.matrix(prob6))
    expect_identical(
        dimnames(prob6),
        NULL
    )
    expect_error(
        probability(wov, c(TRUE, FALSE), mode = "character"),
        "targets must be a character vector or a named numeric vector"
    )
    expect_error(
        probability(wov, c(1, -1), mode = "character"),
        "targets must be named"
    )
    wov$normalize <- TRUE
    expect_error(
        probability(wov, c(1, -1), mode = "character"),
        "x must be trained with normalize = FALSE"
    )
    
    # doc2vec
    expect_error(
        probability(list(), c("us" = 1, "people" = -1)),
        "x must be a textmodel_wordvector object"
    )
})

test_that("get_threads are working", {
    
    options("wordvector_threads" = "abc")
    expect_error(
        suppressWarnings(wordvector:::get_threads()),
        "wordvector_threads must be an integer"
    )
    options("wordvector_threads" = NA)
    expect_error(
        wordvector:::get_threads(),
        "wordvector_threads must be an integer"
    )
    
    ## respect other settings
    options("wordvector_threads" = NULL)
    
    Sys.setenv("OMP_THREAD_LIMIT" = 2)
    expect_equal(
        wordvector:::get_threads(), 2
    )
    Sys.unsetenv("OMP_THREAD_LIMIT")
    
    Sys.setenv("RCPP_PARALLEL_NUM_THREADS" = 3)
    expect_equal(
        wordvector:::get_threads(), 3
    )
    Sys.unsetenv("RCPP_PARALLEL_NUM_THREADS")
    
    options("wordvector_threads" = NULL)
})

test_that("print and as.matrix works with old objects", {

    wov_nn <- readRDS("../data/word2vec_v0.5.1.RDS") 
    expect_identical(dim(as.matrix(wov_nn)), c(5360L, 10L))
    expect_error(as.matrix(wov_nn, layer = "documents"),
                 "'arg' should be \"words\"")
    expect_output(
        print(wov_nn),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 10, min_count = 2, iter = 10, ",
            "    sample = 1, normalize = FALSE)",
            "",
            "10 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    
    wov_nm <- readRDS("../data/word2vec-norm_v0.5.1.RDS")
    expect_identical(dim(as.matrix(wov_nm)), c(5360L, 10L))
    expect_error(as.matrix(wov_nm, layer = "documents"),
                 "'arg' should be \"words\"")
    expect_output(
        print(wov_nm),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 10, min_count = 2, iter = 10, ",
            "    sample = 1, normalize = TRUE)",
            "",
            "10 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    
    dov <- readRDS("../data/doc2vec_v0.5.1.RDS")
    expect_identical(dim(as.matrix(dov)), c(5234L, 10L))
    expect_error(as.matrix(dov, layer = "words"),
                 "x does not have the layer for words")
    expect_output(
        print(dov),
        paste(
            "",
            "Call:",
            "textmodel_doc2vec(x = dfmt, model = wov)",
            "",
            "10 dimensions; 5,234 documents", sep = "\n"), fixed = TRUE
    )
})

test_that("class check functions work as expected", {
    
    # word2vec
    expect_silent(
        wordvector:::check_word2vec(wov)
    )
    expect_error(
        wordvector:::check_word2vec(dov),
        "model must be a trained textmodel_word2vec"
    )
    
    # doc2vec
    expect_silent(
        wordvector:::check_doc2vec(dov)
    )
    expect_error(
        wordvector:::check_doc2vec(wov),
        "model must be a trained textmodel_doc2vec"
    )

})

test_that("old arguments still works", {
    
    expect_type(
        probability(wov, c("us", "people"), mode = "words"),
        "character"
    )
    
    expect_type(
        probability(wov, c("us", "people"), mode = "values"),
        "double"
    )
    
    expect_type(
        similarity(wov, c("us", "people"), mode = "words"),
        "character"
    )
    
    expect_type(
        similarity(wov, c("us", "people"), mode = "values"),
        "double"
    )
    
})

test_that("perplexity works", {
    
    # infrequent words
    word1 <- c("good", "nice", "excellent", "positive", "fortunate", "correct", "superior", 
              "bad", "nasty", "poor", "negative", "unfortunate",  "wrong", "inferior")
    suppressWarnings(
        ppl1 <- perplexity(wov, word1, dfmt)
    )
    expect_type(ppl1, "double")
    
    # frequent words
    word2 <- c("america", "us", "people", "government", "state", "nation", "world", "peace", "public")
    suppressWarnings(
        ppl2 <- perplexity(wov, word2, dfmt)
    )
    expect_type(ppl2, "double")
    
    # tokens_object
    suppressWarnings(
        ppl3 <- perplexity(wov, word2, toks)
    )
    expect_equal(ppl3, ppl2)
    
    expect_error(
        perplexity(wov, c("good" = 1, "bad" = -1), dfmt),
        "targets must be a character vector"
    )
    
    expect_error(
        perplexity(wov, word2, list),
        "data must be a tokens or dfm"
    )
})

library(quanteda)
library(wordvector)

corp <- head(data_corpus_inaugural, 59) 

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE,
               concatenator = " ") %>% 
    tokens_remove(stopwords(), padding = TRUE) %>% 
    tokens_compound(data_dictionary_LSD2015, keep_unigrams = TRUE)

dfmt <- dfm(toks, remove_padding = TRUE) 

wov_nn <- readRDS("../data/word2vec_v0.5.1.RDS") 
wov_nm <- readRDS("../data/word2vec-norm_v0.5.1.RDS")

test_that("upgrade_pre06() works", {
    wov_nn2 <- wordvector:::upgrade_pre06(wov_nn)
    expect_equal(names(wov_nn2$values),
                 "word")
    expect_equal(wov_nn2$type,
                 "cbow")
    wov_nm2 <- wordvector:::upgrade_pre06(wov_nn)
    expect_equal(names(wov_nm2$values),
                 "word")
    expect_equal(wov_nm2$type,
                 "cbow")
})

test_that("as.matrix() works with old objects", {
    
    skip_on_cran()
    
    expect_identical(dim(as.matrix(wov_nn)), c(5360L, 10L))
    expect_error(as.matrix(wov_nn, layer = "documents"),
                 "'arg' should be \"words\"")
    expect_identical(dim(as.matrix(wov_nm)), c(5360L, 10L))
    expect_error(as.matrix(wov_nm, layer = "documents"),
                 "'arg' should be \"words\"")

})

test_that("print() works", {
    
    skip_on_cran()
    
    expect_output(
        print(wov_nn),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 10, min_count = 2, iter = 10, ",
            "    sample = 1, normalize = FALSE)",
            "",
            "10 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    
    expect_output(
        print(wov_nm),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 10, min_count = 2, iter = 10, ",
            "    sample = 1, normalize = TRUE)",
            "",
            "10 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )

})

test_that("as.textmodel_doc2vec() works", {
    
    skip_on_cran()
    
    wov_nn <- readRDS("../data/word2vec_v0.5.1.RDS") 
    expect_identical(
        rownames(as.matrix(as.textmodel_doc2vec(dfmt, wov_nn))),
        docnames(dfmt)
    )
    
    wov_nm <- readRDS("../data/word2vec-norm_v0.5.1.RDS")
    expect_identical(
        rownames(as.matrix(as.textmodel_doc2vec(dfmt, wov_nm))),
        docnames(dfmt)
    )
})

library(quanteda)
library(wordvector)
options(wordvector_threads = 2)

corp <- head(data_corpus_inaugural, 59) %>% 
    corpus_reshape()

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords(), padding = FALSE) 

test_that("textmodel_word2vec works", {
    
    skip_on_cran()
    
    # CBOW
    expect_output(
        wov1 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1, verbose = TRUE),
        "Training continuous BOW model with 50 dimensions"
    )
    expect_equal(
        class(wov1), 
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    expect_true(
        wov1$use_ns
    )
    expect_identical(
        wov1$ns_size, 5L
    )
    expect_identical(
        wov1$window, 5L
    )
    expect_identical(
        dim(wov1$values$word), c(5360L, 50L)
    )
    expect_identical(
        dim(wov1$weights), c(5360L, 50L)
    )
    expect_identical(
        wov1$sample, 1.0
    )
    expect_equal(
        wov1$min_count, 2L
    )
    expect_false(
        wov1$normalize
    )
    expect_identical(
        featfreq(dfm_trim(dfm(toks), 2)),
        wov1$frequency
    )
    expect_true(
        wov1$tolower
    )
    expect_silent(
        textmodel_word2vec(toks[1:100], type = "cbow", verbose = FALSE)
    )
    expect_output(
        print(wov1),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 50, min_count = 2, iter = 10, ",
            "    sample = 1, verbose = TRUE)",
            "",
            "50 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(wov1))), 
        class(wov1)
    )
    
    expect_equal(
        rownames(probability(wov1, c("good", "bad"), layer = "words", mode = "numeric")),
        rownames(wov1$values$word)
    )
    
    expect_error(
        probability(wov1, c("good", "bad"), layer = "documents", mode = "numeric"),
        "textmodel_word2vec does not have the layer for documents"
    )
    
    # SG
    expect_output(
        wov2 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                                   type = "sg", verbose = TRUE),
        "Training skip-gram model with 50 dimensions"
    )
    expect_equal(
        class(wov2), 
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    expect_true(
        wov2$use_ns
    )
    expect_identical(
        wov2$ns_size, 5L
    )
    expect_identical(
        wov2$window, 10L
    )
    expect_identical(
        dim(wov2$values$word), c(5360L, 50L)
    )
    expect_identical(
        dim(wov2$weights), c(5360L, 50L)
    )
    expect_identical(
        wov2$sample, 1.0
    )
    expect_equal(
        wov2$min_count, 2L
    )
    expect_false(
        wov2$normalize
    )
    expect_identical(
        featfreq(dfm_trim(dfm(toks), 2)),
        wov2$frequency
    )
    expect_true(
        wov2$tolower
    )
    expect_silent(
        textmodel_word2vec(toks[1:100], type = "sg", verbose = FALSE)
    )
    expect_output(
        print(wov2),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 50, type = \"sg\", min_count = 2, ",
            "    iter = 10, sample = 1, verbose = TRUE)",
            "",
            "50 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(wov2))), 
        class(wov2)
    )
    
    expect_equal(
        rownames(probability(wov2, c("good", "bad"), layer = "words", mode = "numeric")),
        rownames(wov2$values$word)
    )
    
    expect_error(
        probability(wov2, c("good", "bad"), layer = "documents", mode = "numeric"),
        "textmodel_word2vec does not have the layer for documents"
    )
    
    # DM
    expect_output(
        wov3 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                                   type = "dm", verbose = TRUE),
        "Training distributed memory model with 50 dimensions"
    )
    expect_equal(
        class(wov3), 
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    expect_true(
        wov3$use_ns
    )
    expect_identical(
        wov3$ns_size, 5L
    )
    expect_identical(
        wov3$window, 5L
    )
    expect_identical(
        dim(wov3$values$word), c(5360L, 50L)
    )
    expect_null(
        wov3$values$doc
    )
    expect_identical(
        dim(wov3$weights), c(5360L, 50L)
    )
    expect_identical(
        wov3$sample, 1.0
    )
    expect_equal(
        wov3$min_count, 2L
    )
    expect_false(
        wov3$normalize
    )
    expect_identical(
        featfreq(dfm_trim(dfm(toks), 2)),
        wov3$frequency
    )
    expect_true(
        wov3$tolower
    )
    
    expect_output(
        print(wov3),
        paste(
            "",
            "Call:",
            "textmodel_word2vec(x = toks, dim = 50, type = \"dm\", min_count = 2, ",
            "    iter = 10, sample = 1, verbose = TRUE)",
            "",
            "50 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(wov3))), 
        class(wov3)
    )
    
    expect_equal(
        rownames(probability(wov3, c("good", "bad"), layer = "words", mode = "numeric")),
        rownames(wov3$values$word)
    )
    
    expect_error(
        probability(wov3, c("good", "bad"), layer = "documents", mode = "numeric"),
        "textmodel_word2vec does not have the layer for documents"
    )
})


test_that("textmodel_word2vec works hierachical softmax", {
    
    skip_on_cran()
    
    # CBOW
    wov1 <- textmodel_word2vec(head(toks, 1000), type = "cbow", dim = 10, use_ns = FALSE)
    expect_equal(
        class(wov1), 
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    expect_false(
        wov1$use_ns
    )
    expect_equal(
        wov1$type, 
        "cbow"
    )
    
    # SG
    
    wov2 <- textmodel_word2vec(head(toks, 1000), dim = 10, type = "sg", use_ns = FALSE)
    expect_equal(
        class(wov2), 
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    expect_false(
        wov2$use_ns
    )
    expect_equal(
        wov2$type, 
        "sg"
    )
})

test_that("works with old names of type", {
    
    expect_output(
        wov <- textmodel_word2vec(head(toks, 10), dim = 50, iter = 10, 
                                   type = "skip-gram", verbose = TRUE),
        "Training skip-gram model with 50 dimensions"
    )
    expect_equal(
        wov$window,
        10L
    )
    expect_equal(
        wov$type, 
        "sg"
    )
})

test_that("textmodel_word2vec works with include_data", {
    
    skip_on_cran()
    wov0 <- textmodel_word2vec(toks, dim = 10, iter = 1, min_count = 10, 
                               include_data = TRUE)
    expect_identical(wov0$data, toks)
    
    wov1 <- textmodel_word2vec(as.tokens_xptr(toks), dim = 10, iter = 1, min_count = 10, 
                               include_data = TRUE)
    expect_identical(wov1$data, toks)
    
})

test_that("normalize is defunct", {
    
    skip_on_cran()
    
    expect_error({
        textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                                   normalize = TRUE)
        "'normalize' is defunct."
    })

})

test_that("tolower is working", {
    
    skip_on_cran()
    
    wov0 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                               tolower = FALSE)
    expect_equal(dim(wov0$values$word),
                 c(5556L, 50L))
                 
    
    wov1 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1,
                               tolower = TRUE)
    expect_equal(dim(wov1$values$word),
                 c(5360L, 50L))
    
})

test_that("tokens and tokens_xptr produce the same result", {
    
    skip_on_cran()
    
    set.seed(1234)
    wov0 <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1)

    set.seed(1234)
    xtoks <- as.tokens_xptr(toks)
    wov1 <- textmodel_word2vec(xtoks, dim = 50, iter = 10, min_count = 2, sample = 1)
    
    expect_equal(
        names(wov0),
        names(wov1)
    )
    
    expect_equal(
        dimnames(wov0$values$word), 
        dimnames(wov1$values$word) 
    )

})

test_that("textmodel_word2vec is robust", {
    
    expect_s3_class(
        textmodel_word2vec(head(toks, 1), dim = 50, iter = 10, min_count = 1),
        c("textmodel_word2vec", "textmodel_wordvector")
    )
    
    expect_error(
        suppressWarnings(
            textmodel_word2vec(head(toks, 0), dim = 50, iter = 10, min_count = 1)
        ),
        "Failed to train word2vec"
    )
    
    expect_error(
        suppressWarnings(
            textmodel_word2vec(toks, dim = 0, iter = 10, min_count = 1)
        ),
        "The value of dim must be between 2 and Inf"
    )
    
    expect_error(
        suppressWarnings(
            textmodel_word2vec(toks, dim = 50, iter = 0, min_count = 1)
        ),
        "The value of iter must be between 1 and Inf"
    )
  
})  

library(quanteda)
library(wordvector)
options(wordvector_threads = 2)

corp <- head(data_corpus_inaugural, 59)

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE,
               concatenator = " ") %>% 
    tokens_remove(stopwords(), padding = TRUE) %>% 
    tokens_compound(data_dictionary_LSD2015, keep_unigrams = TRUE)

dfmt <- dfm(toks, remove_padding = TRUE) 

set.seed(1234)
wov <- textmodel_word2vec(toks, dim = 50, iter = 10, min_count = 2, sample = 1)

test_that("textmodel_doc2vec works", {
    
    dov1 <- as.textmodel_doc2vec(dfmt, wov)
    expect_false(dov1$normalize)
    expect_equal(
        names(dov1),
        c("values", "weights", "dim", "tolower", "concatenator", "docvars", 
          "normalize", "call", "version")
    )
    expect_equal(
        dim(dov1$values$word), c(5363L, 50L)
    )
    expect_equal(
        dim(dov1$values$doc), c(59L, 50L)
    )
    expect_equal(
        class(dov1), c("textmodel_doc2vec", "textmodel_wordvector")
    )
    expect_output(
        print(dov1),
        paste(
            "",
            "Call:",
            "as.textmodel_doc2vec(x = dfmt, model = wov)",
            "",
            "50 dimensions; 59 documents.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        rownames(probability(dov1, c("good", "bad"), layer = "words", mode = "numeric")),
        rownames(dov1$values$word)
    )
    expect_equal(
        rownames(probability(dov1, c("good", "bad"), layer = "documents", mode = "numeric")),
        rownames(dov1$values$doc)
    )
    
    # normalize
    dov2 <- as.textmodel_doc2vec(dfmt, wov, normalize = TRUE)
    expect_false(identical(dov1$values, dov2$values))
    expect_true(dov2$normalize)
    
    expect_error(
        probability(dov2, c("good", "bad"), layer = "words"),
        "x must be trained with normalize = FALSE"
    )
    expect_error(
        probability(dov2, c("good", "bad"), layer = "documents"),
        "x must be trained with normalize = FALSE"
    )
    
})

test_that("as.textmodel_doc2vec works with different objects", {
    
    expect_equal(
        class(as.textmodel_doc2vec(dfmt, wov)),
        c("textmodel_doc2vec", "textmodel_wordvector")
    )
    
    expect_error(
        as.textmodel_doc2vec(toks, wov),
        "no applicable method for 'as.textmodel_doc2vec'"
    )
    
    expect_error(
        as.textmodel_doc2vec(dfmt, list()),
        "model must be a trained textmodel_word2vec"
    )
})

test_that("textmodel_doc2vec returns zero for emptry documents (#17)", {
    toks <- tokens(c("Citizens of the United States", "")) %>% 
        tokens_tolower()
    dfmt <- dfm(toks)
    dov <- as.textmodel_doc2vec(dfmt, wov)
    expect_true(all(dov$values$doc[1,] != 0))
    expect_true(all(dov$values$doc[2,] == 0))
})

library(quanteda)
library(wordvector)
library(quanteda.textstats)
options(wordvector_threads = 8)
quanteda_options(verbose = TRUE)

# Load data
dat <- readRDS('~/yahoo-news.RDS')
dat$text <- paste0(dat$head, ". ", dat$body)
corp <- corpus(dat, text_field = 'text', docid_field = "tid")

# Pre-processing
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>% 
    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                  padding = TRUE) %>% 
    tokens_tolower()

wdv <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, iter = 10, alpha = 0.1, verbose = TRUE)
similarity(wdv, analogy(~ washington - america + france)) %>% 
    head()

wdv2 <- textmodel_word2vec(toks, dim = 50, type = "sg", min_count = 5, iter = 10, alpha = 0.1, verbose = TRUE)
similarity(wdv2, analogy(~ washington - america + france)) %>% 
    head()

for (i in 1:10) {
    cat(i, "\n")
    wdv <- textmotel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, verbose = TRUE)
}

similarity(wdv, analogy(~ washington - america + france)) %>% 
    head()
similarity(wdv, analogy(~ berlin - germany + france)) %>% 
    head()

#' Word2vec model
#' 
#' Train a word2vec model (Mikolov et al., 2013) using a [quanteda::tokens] object.
#' @param x a [quanteda::tokens] or [quanteda::tokens_xptr] object.
#' @param dim the size of the word vectors.
#' @param type the architecture of the model; either "cbow" (continuous back-of-words), 
#'   "sg" (skip-gram), or "dm" (distributed memory).
#' @param min_count the minimum frequency of the words. Words less frequent than 
#'   this in `x` are removed before training.
#' @param window the size of the word window. Words within this window are considered 
#'   to be the context of a target word.
#' @param iter the number of iterations in model training.
#' @param alpha the initial learning rate.
#' @param use_ns if `TRUE`, negative sampling is used. Otherwise, hierarchical softmax 
#'   is used.
#' @param ns_size the size of negative samples. Only used when `use_ns = TRUE`.
#' @param sample the rate of sampling of words based on their frequency. Sampling is 
#'   disabled when `sample = 1.0`
#' @param tolower lower-case all the tokens before fitting the model.
#' @param model a trained Word2vec model; if provided, its word vectors are updated for `x`.
#' @param include_data if `TRUE`, the resulting object includes the data supplied as `x`.
#' @param verbose if `TRUE`, print the progress of training.
#' @param ... additional arguments.
#' @returns Returns a textmodel_word2vec object with the following elements:
#'   \item{values}{a list of a matrix for word vector values.}
#'   \item{weights}{a matrix for word vector weights.}
#'   \item{dim}{the size of the word vectors.}
#'   \item{type}{the architecture of the model.}
#'   \item{frequency}{the frequency of words in `x`.}
#'   \item{window}{the size of the word window.}
#'   \item{iter}{the number of iterations in model training.}
#'   \item{alpha}{the initial learning rate.}
#'   \item{use_ns}{the use of negative sampling.}
#'   \item{ns_size}{the size of negative samples.}
#'   \item{min_count}{the value of min_count.}
#'   \item{concatenator}{the concatenator in `x`.}
#'   \item{data}{the original data supplied as `x` if `include_data = TRUE`.}
#'   \item{call}{the command used to execute the function.}
#'   \item{version}{the version of the wordvector package.}
#' @details
#'  If `type = "dm"`, it trains a doc2vec model but saves only 
#'  word vectors to save storage space. [wordvector::textmodel_doc2vec] should be 
#'  used to access document vectors. 
#'     
#'  Users can changed the number of processors used for the parallel computing via
#'  `options(wordvector_threads)`. When the value is large than one, the result 
#'  of every execution becomes slightly different even if `set.seed()` is used because 
#'  parameters are updated in different orders by the processors.
#' 
#' @references 
#'   Mikolov, T., Sutskever, I., Chen, K., Corrado, G., & Dean, J. (2013). 
#'   Distributed Representations of Words and Phrases and their Compositionality. 
#'   https://arxiv.org/abs/1310.4546.
#' @export
#' @examples
#' \donttest{
#' library(quanteda)
#' library(wordvector)
#' 
#' # pre-processing
#' corp <- data_corpus_news2014 
#' toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
#'    tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>% 
#'    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
#'                  padding = TRUE) %>% 
#'    tokens_tolower()
#'
#' # train word2vec
#' wov <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, sample = 0.001)
#'
#' # find similar words
#' head(similarity(wov, c("berlin", "germany", "france"), mode = "words"))
#' head(similarity(wov, c("berlin" = 1, "germany" = -1, "france" = 1), mode = "values"))
#' head(similarity(wov, analogy(~ berlin - germany + france), mode = "words"))
#' }
textmodel_word2vec <- function(x, dim = 50, type = c("cbow", "sg", "dm"), 
                               min_count = 5, window = ifelse(type == "sg", 10, 5), 
                               iter = 10, alpha = 0.05, model = NULL, 
                               use_ns = TRUE, ns_size = 5, sample = 0.001, tolower = TRUE,
                               include_data = FALSE, verbose = FALSE, ...) {
    UseMethod("textmodel_word2vec")
}

#' @import quanteda
#' @useDynLib wordvector
#' @export
#' @method textmodel_word2vec tokens
#' 
textmodel_word2vec.tokens <- function(x, dim = 50, type = c("cbow", "sg", "dm"), 
                               min_count = 5, window = ifelse(type == "sg", 10, 5), 
                               iter = 10, alpha = 0.05, model = NULL, 
                               use_ns = TRUE, ns_size = 5, sample = 0.001, tolower = TRUE,
                               include_data = FALSE, verbose = FALSE, ...) {
    
    type <- ifelse(type == "skip-gram", "sg", type) # for backward compatibility
    type <- match.arg(type)
    wordvector(x, dim, type, FALSE, min_count, window, iter, alpha, model, 
               use_ns, ns_size, sample, tolower, include_data, verbose, ...)
    
}

wordvector <- function(x, dim = 50, type = c("cbow", "sg", "dm", "dbow"), 
                       doc2vec = FALSE, 
                       min_count = 5, window = ifelse(type == "sg", 10, 5), 
                       iter = 10, alpha = 0.05, model = NULL, 
                       use_ns = TRUE, ns_size = 5, sample = 0.001, tolower = TRUE,
                       include_data = FALSE, verbose = FALSE, ..., 
                       normalize = FALSE) {
    
    opt <- quanteda_options("verbose")
    quanteda_options(verbose = FALSE)
    
    type <- match.arg(type)
    dim <- check_integer(dim, min = 2)
    min_count <- check_integer(min_count, min = 0)
    window <- check_integer(window, min = 1)
    iter <- check_integer(iter, min = 1)
    use_ns <- check_logical(use_ns)
    ns_size <- check_integer(ns_size, min_len = 1)
    alpha <- check_double(alpha, min = 0)
    sample <- check_double(sample, min = 0)
    normalize <- check_logical(normalize)
    tolower <- check_logical(tolower)
    include_data <- check_logical(include_data)
    verbose <- check_logical(verbose)
    
    if (normalize)
        .Defunct(msg = "'normalize' is defunct. Use 'as.matrix(x, normalize = TRUE)' instead.")
    
    if (!is.null(model)) {
        model <- upgrade_pre06(model)
        if (doc2vec) {
            model <- check_model(model, c("word2vec", "doc2vec"))
        } else {
            model <- check_model(model, c("word2vec"))
        }
        if (model$dim != dim || model$type != type || model$use_ns != use_ns) {
            dim <- model$dim
            type <- model$type
            use_ns <- model$use_ns
            warning("dim, type and use_na are overwritten by the pre-trained model", 
                    call. = FALSE)
        }
    }
    
    if (include_data)
        y <- as.tokens(x)
    
    x <- as.tokens_xptr(x)
    if (tolower)
        x <- tokens_tolower(x)
    x <- tokens_trim(x, min_termfreq = min_count, termfreq_type = "count")
    
    result <- cpp_word2vec(x, model, size = dim, window = window,
                           sample = sample, withHS = !use_ns, negative = ns_size, 
                           threads = get_threads(), iterations = iter,
                           alpha = alpha, 
                           type = match(type, c("cbow", "sg", "dm", "dbow", "dbow2")), 
                           normalize = FALSE, 
                           doc2vec = doc2vec,
                           verbose = verbose)
    
    if (!is.null(result$message))
        stop("Failed to train word2vec (", result$message, ")")
    
    result$type <- type
    result$min_count <- min_count
    result$tolower <- tolower
    result$concatenator <- meta(x, field = "concatenator", type = "object")
    if (include_data) # NOTE: consider removing
        result$data <- y
    if (doc2vec) {
        result$docvars <- attr(x, "docvars")
        result$ntoken <- ntoken(x, remove_padding = TRUE)
        rownames(result$docvars) <- docnames(x)
        rownames(result$values$doc) <- docnames(x)
    }
    result$call <- try(match.call(sys.function(-2), call = sys.call(-2)), silent = TRUE)
    result$version <- utils::packageVersion("wordvector")
    if (doc2vec) {
        class(result) <- c("textmodel_doc2vec", "textmodel_wordvector")
    } else {
        class(result) <- c("textmodel_word2vec", "textmodel_wordvector")
    }
    
    quanteda_options(verbose = opt) # restore
    return(result)
}

word2vec <- function(...) {
    .Deprecated("textmodel_word2vec")
    textmodel_word2vec(...)
}

#' Print method for trained word vectors
#' @param x for print method, the object to be printed
#' @param ... not used.
#' @method print textmodel_word2vec
#' @keywords internal
#' @return an invisible copy of `x`. 
#' @export
print.textmodel_word2vec <- function(x, ...) {
    x <- upgrade_pre06(x)
    cat("\nCall:\n")
    print(x$call)
    cat("\n", prettyNum(x$dim, big.mark = ","), " dimensions; ",
        prettyNum(nrow(x$values$word), big.mark = ","), " words.",
        "\n", sep = "")
    invisible(x)
}

#' Print method for trained document vectors
#' @param x for print method, the object to be printed
#' @param ... unused
#' @method print textmodel_doc2vec
#' @keywords internal
#' @return an invisible copy of `x`. 
#' @export
print.textmodel_doc2vec <- function(x, ...) {
    x <- upgrade_pre06(x)
    cat("\nCall:\n")
    print(x$call)
    cat("\n", prettyNum(x$dim, big.mark = ","), " dimensions; ",
        prettyNum(nrow(x$values$doc), big.mark = ","), " documents.",
        "\n", sep = "")
    invisible(x)
}

#' Extract word or document vectors
#'
#' Extract word or document vectors from a `textmodel_word2vec` or `textmodel_doc2vec` object.
#' @rdname as.matrix
#' @param x a `textmodel_word2vec` or `textmodel_doc2vec` object.
#' @param normalize if `TRUE`, returns normalized vectors.
#' @param layer the layer from which the vectors are extracted.
#' @param group \[experimental\] average sentence or paragraph vectors from the same document. 
#'   Silently ignored when `layer = "words"`. 
#' @param padding if `TRUE`, add a row with zeros before the word vectors. 
#' @param ... not used.
#' @return a matrix that contain the word or document vectors in rows.
#' @export
as.matrix.textmodel_word2vec <- function(x, normalize = TRUE, 
                                         layer = "words", padding = FALSE, ...){
    
    x <- upgrade_pre06(x)
    layer <- match.arg(layer)
    normalize <- check_logical(normalize)
    padding <- check_logical(padding)
    
    result <- x$values$word
    if (normalize) {
        v <- sqrt(rowSums(result ^ 2) / ncol(result))
        result <- result / v
    }
    if (padding)
        result <- rbind(rep(0, x$dim), result)
    return(result) 
}

# for old objects before v0.6.0

#' @noRd
#' @method print textmodel_docvector
#' @export
print.textmodel_docvector <- print.textmodel_doc2vec

#' @noRd
#' @method print textmodel_wordvector
#' @export
print.textmodel_wordvector <- print.textmodel_word2vec

#' @noRd
#' @export
as.matrix.textmodel_docvector <- as.matrix.textmodel_doc2vec

#' @noRd
#' @export
as.matrix.textmodel_wordvector <- as.matrix.textmodel_word2vec

library(quanteda)
library(wordvector)
library(word2vec)
options(wordvector_threads = 2)

jaccard <- function(a, b) {
    mapply(function(x, y) {length(intersect(x, y)) / length(union(x, y))},
           as.data.frame.matrix(a), as.data.frame.matrix(b)
    )
}

correlation <- function(a, b) {
    mapply(function(x, y) {cor(x, y)},
           as.data.frame.matrix(a), as.data.frame.matrix(b)
    )
}

corp <- head(data_corpus_inaugural, 59) %>% 
    corpus_reshape()

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords(), padding = FALSE) %>% 
    tokens_tolower()
toks_grp <- tokens_group(toks)

lis <- as.list(toks)
lis_grp <- as.list(toks_grp)

feat <- names(topfeatures(dfm(toks), 20))
#feat <- c("people", "government", "us", "world", "country")
doc <- tail(docnames(toks_grp), 20)

test_that("Skip-gram models are similar", {
    skip_on_cran()
    set.seed(1234)
    
    wdv <- textmodel_word2vec(toks, dim = 100, iter = 20, min_count = 0, type = "skip-gram",
                              verbose = FALSE, sample = 0)
    w2v <- word2vec(lis, dim = 100, iter = 20, min_count = 0, type = "skip-gram",
                    verbose = FALSE, threads = 2, sample = 0)
    
    expect_true(all(
        correlation(proxyC::simil(as.matrix(wdv)[feat,]),
                    proxyC::simil(as.matrix(w2v)[feat,])) > 0.8
    ))
    
    dov <- as.matrix(textmodel_doc2vec(toks_grp, wdv))
    d2v <- doc2vec(w2v, newdata = sapply(lis_grp, paste, collapse = " "))
    
    expect_true(all(
        correlation(proxyC::simil(dov[doc,]), 
                    proxyC::simil(d2v[doc,])) > 0.8
    ))
})

test_that("CBOW models are similar", {
    skip_on_cran()
    set.seed(1234)
    
    wdv <- textmodel_word2vec(toks, dim = 100, iter = 20, min_count = 0, type = "cbow",
                              verbose = FALSE, sample = 0)
    w2v <- word2vec(lis, dim = 100, iter = 20, min_count = 0, type = "cbow",
                              verbose = FALSE, threads = 2, sample = 0)
    expect_true(all(
        correlation(proxyC::simil(as.matrix(wdv)[feat,]),
                    proxyC::simil(as.matrix(w2v)[feat,])) > 0.8
    ))
    
    dov <- as.matrix(textmodel_doc2vec(toks_grp, wdv))
    d2v <- doc2vec(w2v, newdata = sapply(lis_grp, paste, collapse = " "))
    
    expect_true(all(
        correlation(proxyC::simil(dov[doc,]), 
                    proxyC::simil(d2v[doc,])) > 0.8
    ))
})

#' Create distributed representation of documents
#' 
#' Create distributed representation of documents as weighted word vectors.
#' @param x a [quanteda::tokens] or [quanteda::dfm] object.
#' @param model a textmodel_wordvector object.
#' @param normalize if `TRUE`, normalized word vectors before creating document vectors.
#' @param group_data if `TRUE`, apply `dfm_group(x)` before creating document vectors.
#' @param ... additional arguments passed to [quanteda::object2id].
#' @returns Returns a textmodel_docvector object with the following elements:
#'   \item{values}{a list of matrices for word and document vectors.}
#'   \item{dim}{the size of the document vectors.}
#'   \item{concatenator}{the concatenator in `x`.}
#'   \item{docvars}{document variables copied from `x`.}
#'   \item{normalize}{if the document vectors are normalized.}
#'   \item{call}{the command used to execute the function.}
#'   \item{version}{the version of the wordvector package.}
#' @export
as.textmodel_doc2vec <- function(x, model, normalize = FALSE, 
                              group_data = FALSE, ...) {
    UseMethod("as.textmodel_doc2vec")
}


#' @export
#' @method as.textmodel_doc2vec dfm
as.textmodel_doc2vec.dfm <- function(x, model = NULL, normalize = FALSE, 
                                     group_data = FALSE, ...) {
    
    model <- upgrade_pre06(model)
    model <- check_model(model, c("word2vec", "lsa"))
    conc <- meta(x, field = "concatenator", type = "object")

    wov <- as.matrix(model, normalize)
    if (group_data)
        x <- dfm_group(x)
    x <- dfm_match(x, rownames(wov))
    
    l <- rowSums(x) == 0
    dov <- as.matrix(Matrix::tcrossprod(x, t(wov))) # NOTE: consider using proxyC::prod
    dov <- dov / sqrt(rowSums(dov ^ 2) / ncol(dov))
    dov[l,] <- 0
    
    result <- list(
        "values" = list("word" = wov, "doc" = dov),
        "weights" = model$weights,
        "dim" = model$dim,
        "tolower" = model$tolower,
        "concatenator" = conc, 
        "docvars" = x@docvars,
        "normalize" = normalize,
        "call" = try(match.call(sys.function(-1), call = sys.call(-1)), silent = TRUE), 
        "version" = utils::packageVersion("wordvector")
    )
    class(result) <- c("textmodel_doc2vec", "textmodel_wordvector")
    return(result)
}

library(quanteda)
library(wordvector)

corp <- head(data_corpus_inaugural, 59) %>% 
    corpus_reshape()

toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
    tokens_remove(stopwords(), padding = FALSE)

dfmt <- dfm(toks, remove_padding = TRUE) 

set.seed(1234)
wov <- textmodel_lsa(toks, dim = 50, min_count = 2, sample = 1)
dov <- as.textmodel_doc2vec(dfmt, wov)
dov_gp <- as.textmodel_doc2vec(dfmt, wov, group_data = TRUE)

test_that("word2vec words", {
    
    # wordvector
    expect_equal(
        class(wov), 
        c("textmodel_lsa", "textmodel_wordvector")
    )
    expect_identical(
        dim(wov$values$word), c(5360L, 50L)
    )
    expect_equal(
        wov$weight, "count"
    )
    expect_identical(
        wov$min_count, 2L
    )
    expect_identical(
        featfreq(dfm_trim(dfm(toks), 2)),
        wov$frequency
    )
    expect_output(
        print(wov),
        paste(
            "",
            "Call:",
            "textmodel_lsa(x = toks, dim = 50, min_count = 2, sample = 1)",
            "",
            "50 dimensions; 5,360 words.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(wov))), 
        class(wov)
    )
    expect_equal(
        names(dov),
        c("values", "weights", "dim", "tolower", "concatenator", "docvars", "normalize", 
          "call", "version")
    )
    
    # docvector with model
    expect_identical(
        dim(dov$values$word), c(5360L, 50L)
    )
    expect_equal(
        dim(dov$values$doc), c(5296L, 50L)
    )
    expect_equal(
        class(dov), 
        c("textmodel_doc2vec", "textmodel_wordvector")
    )
    expect_output(
        print(dov),
        paste(
            "",
            "Call:",
            "as.textmodel_doc2vec(x = dfmt, model = wov)",
            "",
            "50 dimensions; 5,296 documents.", sep = "\n"), fixed = TRUE
    )
    expect_equal(
        class(expect_output(print(dov))), 
        class(dov)
    )
    expect_equal(
        names(dov),
        c("values", "weights", "dim", "tolower", "concatenator", "docvars", "normalize", "call", "version")
    )
    
    # docvector with grouped data
    expect_identical(
        dim(dov_gp$values$word), c(5360L, 50L)
    )
    expect_identical(
        dim(dov_gp$values$doc), c(59L, 50L)
    )
    expect_equal(
        class(dov_gp), 
        c("textmodel_doc2vec", "textmodel_wordvector")
    )
    expect_equal(
        names(dov_gp),
        c("values", "weights", "dim", "tolower", "concatenator", "docvars", "normalize", "call", "version")
    )
})

test_that("tolower is working", {
    
    skip_on_cran()
    
    wov0 <- textmodel_lsa(toks, dim = 50, iter = 10, min_count = 2, 
                          tolower = FALSE)
    expect_equal(dim(wov0$values$word),
                 c(5556L, 50L))
    
    
    wov1 <- textmodel_lsa(toks, dim = 50, iter = 10, min_count = 2, 
                          tolower = TRUE)
    expect_equal(dim(wov1$values$word),
                 c(5360L, 50L))
    
})

#' Yahoo News summaries from 2014
#'
#' A corpus object containing 2,000 news summaries collected from Yahoo News via
#' RSS feeds in 2014. The title and description of the summaries are concatenated.
#' @name data_corpus_news2014
#' @references Watanabe, K. (2018). Newsmap: A semi-supervised approach to
#' geographical news classification. Digital Journalism, 6(3), 294–309.
#' https://doi.org/10.1080/21670811.2017.1293487
#' @source <https://www.yahoo.com/news/>
"data_corpus_news2014"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cpp_get_max_thread <- function() {
    .Call('_wordvector_cpp_get_max_thread', PACKAGE = 'wordvector')
}

cpp_word2vec <- function(xptr, model, size = 100L, window = 5L, sample = 0.001, withHS = FALSE, negative = 5L, threads = 1L, iterations = 5L, alpha = 0.05, type = 1L, doc2vec = FALSE, verbose = FALSE, normalize = TRUE) {
    .Call('_wordvector_cpp_word2vec', PACKAGE = 'wordvector', xptr, model, size, window, sample, withHS, negative, threads, iterations, alpha, type, doc2vec, verbose, normalize)
}

library(quanteda)
library(wordvector)
options(wordvector_threads = 8)
quanteda_options(verbose = TRUE)

corp <- data_corpus_news2014
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>%
    tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>%
    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                 padding = TRUE) %>%
    tokens_tolower()

wdv <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, iter = 10)
similarity(wdv, analogy(~ washington - america + france)) %>% 
    head()

wdv2 <- textmodel_word2vec(toks[1:1000], dim = 50, type = "cbow", min_count = 2, iter = 10, 
                           model = wdv, update_weights = TRUE, verbose = TRUE)
wdv3 <- textmodel_word2vec(toks[1:1000], dim = 50, type = "cbow", min_count = 2, iter = 10, 
                           model = wdv, update_weights = FALSE, verbose = TRUE)


similarity(wdv, analogy(~ washington - america + france)) %>% 
    head()
similarity(wdv2, analogy(~ washington - america + france)) %>% 
    head()
similarity(wdv3, analogy(~ washington - america + france)) %>% 
    head()

wdv$weights["america",]
wdv2$weights["america",]
wdv3$weights["america",]

dov <- textmodel_doc2vec(toks, dim = 100, type = "dm", min_count = 5, iter = 10, verbose = TRUE)
dov2 <- textmodel_doc2vec(toks, dim = 100, type = "dm", min_count = 5, iter = 20, verbose = TRUE)

sim <- proxyC::simil(
    dov$values$doc,
    dov$values$doc["3555430",, drop = FALSE]
)
tail(sort(s <- rowSums(sim)))

sim2 <- proxyC::simil(
    dov2$values$doc,
    dov2$values$doc["3555430",, drop = FALSE]
)
tail(sort(s2 <- rowSums(sim2)))

identical(s, s2)
cor(s, s2)

sim <- proxyC::simil(
    dov2$values$doc,
    dov2$values$doc["4263794",, drop = FALSE]
)
sim <- proxyC::simil(
    dov2$values$doc,
    dov2$values$doc["3016236",, drop = FALSE]
)
sim <- proxyC::simil(
    dov2$values$doc,
    dov2$values$doc["3555430",, drop = FALSE]
)
tail(sort(s <- rowSums(sim)))
print(tail(toks[order(s)]), max_ntoken = -1)

for (i in 1:10) {
    cat(i, "\n")
    wdv <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, iter = 30, verbose = TRUE)
}

# word2vec -------------------------------------
wdv <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, iter = 5, 
                          verbose = TRUE)

similarity(wdv, analogy(~ washington - america + france)) %>% 
    head()
similarity(wdv, analogy(~ berlin - germany + france)) %>% 
    head()

# -----------------------------

dov <- textmodel_doc2vec(toks, dim = 50, type = "skip-gram", min_count = 5, iter = 10, verbose = TRUE)
seed <- LSX::seedwords("sentiment")
doc <- probability(dov, seed, layer = "documents")[,1]
print(toks[head(doc)], -1, -1)
print(toks[tail(doc)], -1, -1)

# LSA -------------------------------------

lsa <- textmodel_lsa(toks, dim = 50, min_count = 0, verbose = TRUE)
similarity(wdv, analogy(~ berlin - germany + france)) %>% 
    head()

#' Latent Semantic Analysis model
#' 
#' Train a Latent Semantic Analysis model (Deerwester et al., 1990) on a [quanteda::tokens] object.
#' @param x a [quanteda::tokens] or [quanteda::tokens_xptr] object.
#' @param dim the size of the word vectors.
#' @param min_count the minimum frequency of the words. Words less frequent than 
#'    this in `x` are removed before training.
#' @param engine select the engine perform SVD to generate word vectors.
#' @param weight weighting scheme passed to [quanteda::dfm_weight()]. 
#' @param tolower if `TRUE` lower-case all the tokens before fitting the model.
#' @param verbose if `TRUE`, print the progress of training.
#' @param ... additional arguments.
#' @returns Returns a textmodel_wordvector object with the following elements:
#'   \item{values}{a matrix for word vectors values.}
#'   \item{weights}{a matrix for word vectors weights.}
#'   \item{frequency}{the frequency of words in `x`.}
#'   \item{engine}{the SVD engine used.}
#'   \item{weight}{weighting scheme.}
#'   \item{min_count}{the value of min_count.}
#'   \item{concatenator}{the concatenator in `x`.}
#'   \item{call}{the command used to execute the function.}
#'   \item{version}{the version of the wordvector package.}
#' @references 
#'   Deerwester, S. C., Dumais, S. T., Landauer, T. K., Furnas, G. W., & Harshman, R. A. (1990). 
#'   Indexing by latent semantic analysis. JASIS, 41(6), 391–407.
#' @export
#' @examples
#' \donttest{
#' library(quanteda)
#' library(wordvector)
#' 
#' # pre-processing
#' corp <- corpus_reshape(data_corpus_news2014)
#' toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) %>% 
#'    tokens_remove(stopwords("en", "marimo"), padding = TRUE) %>% 
#'    tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
#'                  padding = TRUE) %>% 
#'    tokens_tolower()
#' 
#' # train LSA
#' lsa <- textmodel_lsa(toks, dim = 50, min_count = 5, verbose = TRUE)
#' 
#' # find similar words
#' head(similarity(lsa, c("berlin", "germany", "france"), mode = "words"))
#' head(similarity(lsa, c("berlin" = 1, "germany" = -1, "france" = 1), mode = "values"))
#' head(similarity(lsa, analogy(~ berlin - germany + france)))
#' }
textmodel_lsa <- function(x, dim = 50, min_count = 5L, 
                          engine = c("RSpectra", "irlba", "rsvd"), 
                          weight = "count", tolower = TRUE, verbose = FALSE, ...) {
    UseMethod("textmodel_lsa")   
}

#' @import quanteda
#' @export
#' @method textmodel_lsa tokens
textmodel_lsa.tokens <- function(x, dim = 50L, min_count = 5L, 
                                 engine = c("RSpectra", "irlba", "rsvd"), 
                                 weight = "count", tolower = TRUE, verbose = FALSE, ...) {
    
    result <- textmodel_lsa(dfm(x, remove_padding = TRUE, tolower = tolower), 
                            dim = dim, min_count = min_count, engine = engine, weight = weight,
                            verbose = verbose, ...)
    result$call = try(match.call(sys.function(-1), call = sys.call(-1)), silent = TRUE)
    return(result)
}

#' @import quanteda
#' @export
#' @method textmodel_lsa dfm
textmodel_lsa.dfm <- function(x, dim = 50L, min_count = 5L, 
                                 engine = c("RSpectra", "irlba", "rsvd"), 
                                 weight = "count", tolower = TRUE, verbose = FALSE, ...) {
    
    engine <- match.arg(engine)
    dim <- check_integer(dim, min = 2)
    min_count <- check_integer(min_count, min = 0)
    verbose <- check_logical(verbose)
    
    x <- dfm_trim(x, min_termfreq = min_count, termfreq_type = "count")
    if (engine %in% c("RSpectra", "irlba", "rsvd")) {
        if (verbose) {
            cat(sprintf("Performing SVD into %d dimensions\n", dim))
            cat(sprintf("...using %s\n", engine))
        }
        svd <- get_svd(x, dim, engine, weight, ...)
        if (verbose)
            cat("...complete\n")
        wov <- svd$v
        rownames(wov) <- featnames(x)
    }
    result <- list(
        "values" = list(word = wov),
        "dim" = dim,
        "frequency" = featfreq(x),
        "engine" = engine,
        "weight" = weight,
        "min_count" = min_count,
        "tolower" = tolower,
        "concatenator" = meta(x, field = "concatenator", type = "object"),
        "call" = try(match.call(sys.function(-1), call = sys.call(-1)), silent = TRUE),
        "version" = utils::packageVersion("wordvector")
    )
    class(result) <- c("textmodel_lsa", "textmodel_wordvector")
    return(result)
}

#' @importFrom quanteda dfm_weight
#' @importFrom methods as
get_svd <- function(x, k, engine, weight = "count", reduce = FALSE, ...) {
    if (reduce) {
        # NOTE: generalize? featfreq(x) ^ (1 / reduce)
        x <- dfm_weight(x, weights = 1 / sqrt(featfreq(x)))
    } else {
        if (weight == "sqrt") {
            x@x <- sqrt(x@x)
        } else {
            x <- dfm_weight(x, scheme = weight)
        }
    }
    if (engine == "RSpectra") {
        result <- RSpectra::svds(as(x, "dgCMatrix"), k = k, nu = 0, nv = k, ...)
    } else if (engine == "rsvd") {
        result <- rsvd::rsvd(as(x, "dgCMatrix"), k = k, nu = 0, nv = k, ...)
    } else {
        result <- irlba::irlba(as(x, "dgCMatrix"), nv = k, right_only = TRUE, ...)
    }
    return(result)
}

lsa <- function(...) {
    .Deprecated("textmodel_lsa")
    textmodel_lsa(...)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{data_corpus_news2014}
\alias{data_corpus_news2014}
\title{Yahoo News summaries from 2014}
\format{
An object of class \code{corpus} (inherits from \code{character}) of length 20000.
}
\source{
\url{https://www.yahoo.com/news/}
}
\usage{
data_corpus_news2014
}
\description{
A corpus object containing 2,000 news summaries collected from Yahoo News via
RSS feeds in 2014. The title and description of the summaries are concatenated.
}
\references{
Watanabe, K. (2018). Newsmap: A semi-supervised approach to
geographical news classification. Digital Journalism, 6(3), 294–309.
https://doi.org/10.1080/21670811.2017.1293487
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{similarity}
\alias{similarity}
\title{Compute similarity between word or document vectors}
\usage{
similarity(
  x,
  targets,
  layer = c("words", "documents"),
  mode = c("character", "numeric")
)
}
\arguments{
\item{x}{a \code{textmodel_wordvector} object.}

\item{targets}{words or documents for which similarity is computed.}

\item{layer}{the layer based on which similarity is computed. This must be "documents"
when \code{targets} are document names.}

\item{mode}{specify the type of resulting object.}
}
\value{
a \code{matrix} of cosine similarity scores when \code{mode = "numeric"} or of
words sorted in descending order by the similarity scores when \code{mode = "character"}.
When \code{targets} is a named numeric vector, word (or document) vectors are weighted and summed
before computing similarity scores.
}
\description{
Compute the cosine similarity between word vectors for selected words.
}
\seealso{
\code{\link[=probability]{probability()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsa.R
\name{textmodel_lsa}
\alias{textmodel_lsa}
\title{Latent Semantic Analysis model}
\usage{
textmodel_lsa(
  x,
  dim = 50,
  min_count = 5L,
  engine = c("RSpectra", "irlba", "rsvd"),
  weight = "count",
  tolower = TRUE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{a \link[quanteda:tokens]{quanteda::tokens} or \link[quanteda:tokens_xptr]{quanteda::tokens_xptr} object.}

\item{dim}{the size of the word vectors.}

\item{min_count}{the minimum frequency of the words. Words less frequent than
this in \code{x} are removed before training.}

\item{engine}{select the engine perform SVD to generate word vectors.}

\item{weight}{weighting scheme passed to \code{\link[quanteda:dfm_weight]{quanteda::dfm_weight()}}.}

\item{tolower}{if \code{TRUE} lower-case all the tokens before fitting the model.}

\item{verbose}{if \code{TRUE}, print the progress of training.}

\item{...}{additional arguments.}
}
\value{
Returns a textmodel_wordvector object with the following elements:
\item{values}{a matrix for word vectors values.}
\item{weights}{a matrix for word vectors weights.}
\item{frequency}{the frequency of words in \code{x}.}
\item{engine}{the SVD engine used.}
\item{weight}{weighting scheme.}
\item{min_count}{the value of min_count.}
\item{concatenator}{the concatenator in \code{x}.}
\item{call}{the command used to execute the function.}
\item{version}{the version of the wordvector package.}
}
\description{
Train a Latent Semantic Analysis model (Deerwester et al., 1990) on a \link[quanteda:tokens]{quanteda::tokens} object.
}
\examples{
\donttest{
library(quanteda)
library(wordvector)

# pre-processing
corp <- corpus_reshape(data_corpus_news2014)
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) \%>\% 
   tokens_remove(stopwords("en", "marimo"), padding = TRUE) \%>\% 
   tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                 padding = TRUE) \%>\% 
   tokens_tolower()

# train LSA
lsa <- textmodel_lsa(toks, dim = 50, min_count = 5, verbose = TRUE)

# find similar words
head(similarity(lsa, c("berlin", "germany", "france"), mode = "words"))
head(similarity(lsa, c("berlin" = 1, "germany" = -1, "france" = 1), mode = "values"))
head(similarity(lsa, analogy(~ berlin - germany + france)))
}
}
\references{
Deerwester, S. C., Dumais, S. T., Landauer, T. K., Furnas, G. W., & Harshman, R. A. (1990).
Indexing by latent semantic analysis. JASIS, 41(6), 391–407.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.doc2vec.R
\name{as.textmodel_doc2vec}
\alias{as.textmodel_doc2vec}
\title{Create distributed representation of documents}
\usage{
as.textmodel_doc2vec(x, model, normalize = FALSE, group_data = FALSE, ...)
}
\arguments{
\item{x}{a \link[quanteda:tokens]{quanteda::tokens} or \link[quanteda:dfm]{quanteda::dfm} object.}

\item{model}{a textmodel_wordvector object.}

\item{normalize}{if \code{TRUE}, normalized word vectors before creating document vectors.}

\item{group_data}{if \code{TRUE}, apply \code{dfm_group(x)} before creating document vectors.}

\item{...}{additional arguments passed to \link[quanteda:object2id]{quanteda::object2id}.}
}
\value{
Returns a textmodel_docvector object with the following elements:
\item{values}{a list of matrices for word and document vectors.}
\item{dim}{the size of the document vectors.}
\item{concatenator}{the concatenator in \code{x}.}
\item{docvars}{document variables copied from \code{x}.}
\item{normalize}{if the document vectors are normalized.}
\item{call}{the command used to execute the function.}
\item{version}{the version of the wordvector package.}
}
\description{
Create distributed representation of documents as weighted word vectors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{perplexity}
\alias{perplexity}
\title{[experimental] Compute perplexity of a model}
\usage{
perplexity(x, targets, data, layer = c("words", "documents"))
}
\arguments{
\item{x}{a trained \code{textmodel_wordvector} object.}

\item{targets}{words for which probabilities are computed.}

\item{data}{a \link[quanteda:tokens]{quanteda::tokens} or \link[quanteda:dfm]{quanteda::dfm}; the probabilities of words are
tested against occurrences of words in it.}

\item{layer}{the layer based on which probabilities are computed.}
}
\description{
Compute the perplexity of a trained word2vec model with data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{probability}
\alias{probability}
\title{Compute probability of words}
\usage{
probability(
  x,
  targets,
  layer = c("words", "documents"),
  mode = c("character", "numeric"),
  ...
)
}
\arguments{
\item{x}{a trained \code{textmodel_wordvector} object.}

\item{targets}{words for which probabilities are computed.}

\item{layer}{the layer based on which probabilities are computed.}

\item{mode}{specify the type of resulting object.}

\item{...}{passed to \code{as.matrix()}.}
}
\value{
a matrix of words or documents sorted in descending order by the probability
scores when \code{mode = "character"}; a matrix of the probability scores when \code{mode = "numeric"}.
When \code{targets} is a named numeric vector, probability scores are weighted by
the values.
}
\description{
Compute the probability of words given other words.
}
\seealso{
\code{\link[=similarity]{similarity()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/word2vec.R
\name{print.textmodel_word2vec}
\alias{print.textmodel_word2vec}
\title{Print method for trained word vectors}
\usage{
\method{print}{textmodel_word2vec}(x, ...)
}
\arguments{
\item{x}{for print method, the object to be printed}

\item{...}{not used.}
}
\value{
an invisible copy of \code{x}.
}
\description{
Print method for trained word vectors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc2vec.R
\name{textmodel_doc2vec}
\alias{textmodel_doc2vec}
\title{Doc2vec model}
\usage{
textmodel_doc2vec(
  x,
  dim = 50,
  type = c("dm", "dbow"),
  min_count = 5,
  window = 5,
  iter = 10,
  alpha = 0.05,
  model = NULL,
  use_ns = TRUE,
  ns_size = 5,
  sample = 0.001,
  tolower = TRUE,
  include_data = FALSE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{a \link[quanteda:tokens]{quanteda::tokens} or \link[quanteda:tokens_xptr]{quanteda::tokens_xptr} object.}

\item{dim}{the size of the word vectors.}

\item{type}{the architecture of the model; either "dm" (distributed memory) or
"dbow" (distributed bag-of-words).}

\item{min_count}{the minimum frequency of the words. Words less frequent than
this in \code{x} are removed before training.}

\item{window}{the size of the window for context words. Ignored when \code{type = "dbow"} as
its context window is the entire document (sentence or paragraph).}

\item{iter}{the number of iterations in model training.}

\item{alpha}{the initial learning rate.}

\item{model}{a trained Word2vec model; if provided, its word vectors are updated for \code{x}.}

\item{use_ns}{if \code{TRUE}, negative sampling is used. Otherwise, hierarchical softmax
is used.}

\item{ns_size}{the size of negative samples. Only used when \code{use_ns = TRUE}.}

\item{sample}{the rate of sampling of words based on their frequency. Sampling is
disabled when \code{sample = 1.0}}

\item{tolower}{lower-case all the tokens before fitting the model.}

\item{include_data}{if \code{TRUE}, the resulting object includes the data supplied as \code{x}.}

\item{verbose}{if \code{TRUE}, print the progress of training.}

\item{...}{additional arguments.}
}
\value{
Returns a textmodel_doc2vec object with matrices for word and document vector
values, \link[quanteda:docvars]{quanteda::docvars} and \link[quanteda:ntoken]{quanteda::ntoken} of \code{x}. Other elements are
the same as \link{textmodel_word2vec}.
}
\description{
Train a doc2vec model (Le & Mikolov, 2014) using a \link[quanteda:tokens]{quanteda::tokens} object.
}
\references{
Le, Q. V., & Mikolov, T. (2014). Distributed Representations of Sentences and
Documents (No. arXiv:1405.4053). arXiv. https://doi.org/10.48550/arXiv.1405.4053
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc2vec.R, R/word2vec.R
\name{as.matrix.textmodel_doc2vec}
\alias{as.matrix.textmodel_doc2vec}
\alias{as.matrix.textmodel_word2vec}
\title{Extract word or document vectors}
\usage{
\method{as.matrix}{textmodel_doc2vec}(
  x,
  normalize = TRUE,
  layer = c("documents", "words"),
  group = FALSE,
  ...
)

\method{as.matrix}{textmodel_word2vec}(x, normalize = TRUE, layer = "words", padding = FALSE, ...)
}
\arguments{
\item{x}{a \code{textmodel_word2vec} or \code{textmodel_doc2vec} object.}

\item{normalize}{if \code{TRUE}, returns normalized vectors.}

\item{layer}{the layer from which the vectors are extracted.}

\item{group}{[experimental] average sentence or paragraph vectors from the same document.
Silently ignored when \code{layer = "words"}.}

\item{...}{not used.}

\item{padding}{if \code{TRUE}, add a row with zeros before the word vectors.}
}
\value{
a matrix that contain the word or document vectors in rows.
}
\description{
Extract word or document vectors from a \code{textmodel_word2vec} or \code{textmodel_doc2vec} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/word2vec.R
\name{textmodel_word2vec}
\alias{textmodel_word2vec}
\title{Word2vec model}
\usage{
textmodel_word2vec(
  x,
  dim = 50,
  type = c("cbow", "sg", "dm"),
  min_count = 5,
  window = ifelse(type == "sg", 10, 5),
  iter = 10,
  alpha = 0.05,
  model = NULL,
  use_ns = TRUE,
  ns_size = 5,
  sample = 0.001,
  tolower = TRUE,
  include_data = FALSE,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{x}{a \link[quanteda:tokens]{quanteda::tokens} or \link[quanteda:tokens_xptr]{quanteda::tokens_xptr} object.}

\item{dim}{the size of the word vectors.}

\item{type}{the architecture of the model; either "cbow" (continuous back-of-words),
"sg" (skip-gram), or "dm" (distributed memory).}

\item{min_count}{the minimum frequency of the words. Words less frequent than
this in \code{x} are removed before training.}

\item{window}{the size of the word window. Words within this window are considered
to be the context of a target word.}

\item{iter}{the number of iterations in model training.}

\item{alpha}{the initial learning rate.}

\item{model}{a trained Word2vec model; if provided, its word vectors are updated for \code{x}.}

\item{use_ns}{if \code{TRUE}, negative sampling is used. Otherwise, hierarchical softmax
is used.}

\item{ns_size}{the size of negative samples. Only used when \code{use_ns = TRUE}.}

\item{sample}{the rate of sampling of words based on their frequency. Sampling is
disabled when \code{sample = 1.0}}

\item{tolower}{lower-case all the tokens before fitting the model.}

\item{include_data}{if \code{TRUE}, the resulting object includes the data supplied as \code{x}.}

\item{verbose}{if \code{TRUE}, print the progress of training.}

\item{...}{additional arguments.}
}
\value{
Returns a textmodel_word2vec object with the following elements:
\item{values}{a list of a matrix for word vector values.}
\item{weights}{a matrix for word vector weights.}
\item{dim}{the size of the word vectors.}
\item{type}{the architecture of the model.}
\item{frequency}{the frequency of words in \code{x}.}
\item{window}{the size of the word window.}
\item{iter}{the number of iterations in model training.}
\item{alpha}{the initial learning rate.}
\item{use_ns}{the use of negative sampling.}
\item{ns_size}{the size of negative samples.}
\item{min_count}{the value of min_count.}
\item{concatenator}{the concatenator in \code{x}.}
\item{data}{the original data supplied as \code{x} if \code{include_data = TRUE}.}
\item{call}{the command used to execute the function.}
\item{version}{the version of the wordvector package.}
}
\description{
Train a word2vec model (Mikolov et al., 2013) using a \link[quanteda:tokens]{quanteda::tokens} object.
}
\details{
If \code{type = "dm"}, it trains a doc2vec model but saves only
word vectors to save storage space. \link{textmodel_doc2vec} should be
used to access document vectors.

Users can changed the number of processors used for the parallel computing via
\code{options(wordvector_threads)}. When the value is large than one, the result
of every execution becomes slightly different even if \code{set.seed()} is used because
parameters are updated in different orders by the processors.
}
\examples{
\donttest{
library(quanteda)
library(wordvector)

# pre-processing
corp <- data_corpus_news2014 
toks <- tokens(corp, remove_punct = TRUE, remove_symbols = TRUE) \%>\% 
   tokens_remove(stopwords("en", "marimo"), padding = TRUE) \%>\% 
   tokens_select("^[a-zA-Z-]+$", valuetype = "regex", case_insensitive = FALSE,
                 padding = TRUE) \%>\% 
   tokens_tolower()

# train word2vec
wov <- textmodel_word2vec(toks, dim = 50, type = "cbow", min_count = 5, sample = 0.001)

# find similar words
head(similarity(wov, c("berlin", "germany", "france"), mode = "words"))
head(similarity(wov, c("berlin" = 1, "germany" = -1, "france" = 1), mode = "values"))
head(similarity(wov, analogy(~ berlin - germany + france), mode = "words"))
}
}
\references{
Mikolov, T., Sutskever, I., Chen, K., Corrado, G., & Dean, J. (2013).
Distributed Representations of Words and Phrases and their Compositionality.
https://arxiv.org/abs/1310.4546.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{analogy}
\alias{analogy}
\title{Convert formula to named character vector}
\usage{
analogy(formula)
}
\arguments{
\item{formula}{a \link{formula} object that defines the relationship between words
using \code{+} or \code{-} operators.}
}
\value{
a named character vector to be passed to \code{\link[=similarity]{similarity()}}.
}
\description{
Convert a formula to a named character vector in analogy tasks.
}
\examples{
analogy(~ berlin - germany + france)
analogy(~ quick - quickly + slowly)
}
\seealso{
\code{\link[=similarity]{similarity()}}
}
